#!/usr/bin/env python
'''
This file contains Python command example for the CoS module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different CoS attributes.
This example is supported on Spectrum devices. 
'''
import sys
import errno, sys
import colorsys
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
from sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)


""" ############################################################################################ """
def get_cos_port_ets_element(count):
    #sx_api_cos_port_ets_element_get 0x12f00 35  
    """ COS PORT ETS ELEMENT GET"""
    print ("--------------- COS PORT ETS ELEMENT GET ------------------------------")
    
    log_port = 0x12f00 
    
    element_count_p = new_uint32_t_p()
    """ Set cnt = 8 """
    uint32_t_p_assign(element_count_p, count) 
    """ Get value from PTR """    
    element_count = uint32_t_p_value(element_count_p)
    print ("element_count (returned from API = %d)" % (element_count))
    
    ets_element_p =  new_sx_cos_ets_element_config_t_arr(element_count)
    
    rc = sx_api_cos_port_ets_element_get(handle,log_port,ets_element_p,element_count_p)
    print("rc = %d" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("sx_api_cos_port_ets_element_get for log_port=0x%x " % (log_port) ) 
    
    
    for i in range(0,element_count):
        ets_element = sx_cos_ets_element_config_t_arr_getitem(ets_element_p, i)   
    
        print("---> %d <---------------" % (i))
        print("element hierarchy=%d " % (ets_element.element_hierarchy))
        print("element index=%d " % (ets_element.element_index))
        print("element next index=%d " % (ets_element.next_element_index))
        print("min shaper enable=%r " % (ets_element.min_shaper_enable))
        print("packets mode=%r " % (ets_element.packets_mode))
        print("min shaper rate=%d " % (ets_element.min_shaper_rate))
        print("max shaper enable=%r " % (ets_element.max_shaper_enable))
        print("max shaper rate=%d " % (ets_element.max_shaper_rate))
        print("dwrr enable=%r " % (ets_element.dwrr_enable))
        print("dwrr=%r " % (ets_element.dwrr))
        print("dwrr_weight=%d " % (ets_element.dwrr_weight))
       

""" ############################################################################################ """

def set_cos_port_ets_element(hierarchy, element_index, next_element_index, 
                             min_shaper_enable, packets_mode,
                             min_shaper_rate, max_shaper_enable, 
                             max_shaper_rate,dwrr_enable, dwrr,
                             dwrr_weight, count ):
    #sx_api_cos_port_ets_element_set 0x12f00 SX_ACCESS_CMD_EDIT 0x12f00 
    #[{3;0;0;0;0;0;1;2000000;0;0;0},{3;1;1;0;0;0;1;2000000;0;0;0},{3;2;2;0;0;0;1;2000000;0;0;0}] 3
    """ COS PORT ETS ELEMENT SET"""
    print ("--------------- COS PORT ETS ELEMENT SET ------------------------------")
    
    log_port = 0x12f00
    cmd =  SX_ACCESS_CMD_ADD
    
    element_count_p = new_uint32_t_p()
    """ Set cnt = 3 """
    uint32_t_p_assign(element_count_p, count) 
    """ Get value from PTR """    
    element_count = uint32_t_p_value(element_count_p)
    print ("element_count (returned from API = %d)" % (element_count))
    
    ets_element_p =  new_sx_cos_ets_element_config_t_arr(element_count)
    
    for i in range(0,element_count):
        ets_element = sx_cos_ets_element_config_t_arr_getitem(ets_element_p, i)
        
        ets_element.element_hierarchy = hierarchy
        ets_element.element_index = element_index + i 
        if hierarchy == 3:
            ets_element.next_element_index = next_element_index + i
        if hierarchy == 2: 
            ets_element.next_element_index = next_element_index
        if hierarchy == 0:
           ets_element.next_element_index = next_element_index
        ets_element.min_shaper_enable = min_shaper_enable
        ets_element.packets_mode = packets_mode
        ets_element.min_shaper_rate = min_shaper_rate
        ets_element.max_shaper_enable = max_shaper_enable
        ets_element.max_shaper_rate = max_shaper_rate
        ets_element.dwrr_enable = dwrr_enable
        ets_element.dwrr = dwrr
        ets_element.dwrr_weight = dwrr_weight    
        
        sx_cos_ets_element_config_t_arr_setitem(ets_element_p, i,ets_element)
    
    rc = sx_api_cos_port_ets_element_set(handle,cmd,log_port,ets_element_p,element_count)
    print("rc = %d" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("sx_api_cos_port_ets_element_set for log_port=0x%x " % (log_port) ) 
    
    
    for i in range(0,element_count):
        ets_element = sx_cos_ets_element_config_t_arr_getitem(ets_element_p, i)   
    
        print("---> %d <---------------" % (i))
        print("element hierarchy=%d " % (ets_element.element_hierarchy))
        print("element index=%d " % (ets_element.element_index))
        print("element next index=%d " % (ets_element.next_element_index))
        print("min shaper enable=%r " % (ets_element.min_shaper_enable))
        print("packets mode=%r " % (ets_element.packets_mode))
        print("min shaper rate=%d " % (ets_element.min_shaper_rate))
        print("max shaper enable=%r " % (ets_element.max_shaper_enable))
        print("max shaper rate=%d " % (ets_element.max_shaper_rate))
        print("dwrr enable=%r " % (ets_element.dwrr_enable))
        print("dwrr=%r " % (ets_element.dwrr))
        print("dwrr_weight=%d " % (ets_element.dwrr_weight))
           

""" ############################################################################################ """
def delete_cos_port_ets_element():
    #sx_api_cos_port_ets_element_set 0x12f00 SX_ACCESS_CMD_DESTROY 0x12f00 [] 0 
    
    """ COS PORT ETS ELEMENT SET - DESTOY"""
    print ("--------------- COS PORT ETS ELEMENT SET - DESTROY------------------------------")
    
    log_port = 0x12f00
    cmd =  SX_ACCESS_CMD_DESTROY
    
    element_count_p = new_uint32_t_p()
    """ Set cnt = 0 """
    uint32_t_p_assign(element_count_p, 0) 
    """ Get value from PTR """    
    
    element_count = uint32_t_p_value(element_count_p)
    print ("element_count (returned from API = %d)" % (element_count))
    
    ets_element_p =  new_sx_cos_ets_element_config_t_arr(element_count)
                
    rc = sx_api_cos_port_ets_element_set(handle,cmd,log_port,ets_element_p,element_count)
    print("rc = %d" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("sx_api_cos_port_ets_element_set - destroy for log_port=0x%x " % (log_port) ) 


""" ############################################################################################ """

get_cos_port_ets_element(35)
set_cos_port_ets_element(3, 0, 0, False, False, 0, True, 2000000, False, False, 0, 3)
get_cos_port_ets_element(35)
delete_cos_port_ets_element()
get_cos_port_ets_element(35)
set_cos_port_ets_element(2, 1, 0, False, False, 0, True, 0, True, True, 20, 5)
get_cos_port_ets_element(35)
delete_cos_port_ets_element()
get_cos_port_ets_element(35)
set_cos_port_ets_element(0, 0, 0, False, False, 0, False, 0, False, False, 0, 1)
get_cos_port_ets_element(35)
delete_cos_port_ets_element()
get_cos_port_ets_element(35)

""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
