#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys
import errno, sys
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
from sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)


""" PORT TYPE BUFF SET """
print ("---------------  PORT TYPE BUFF SET (MIN) ------------------------------")
count = 1
log_port = 0x10100
""" CMD_SET = 15 """
cmd = 15
""" CMD_DELETE = 3 """
""" cmd = 3 """
port_buffer_attr_cnt = count
port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)

for i in range(0,port_buffer_attr_cnt):
    attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
    """ Set type and relevant parameters """
    attr_item_min.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
    
    print ("[%d] type = %d" % ( i, attr_item_min.type) )
    if attr_item_min.type == SX_COS_INGRESS_PORT_ATTR_E:
        attr_item_min.attr.ingress_port_buff_attr.size = 15
        attr_item_min.attr.ingress_port_buff_attr.pool_id = 0
        print ("[%d] Size  = %d" % ( i, attr_item_min.attr.ingress_port_buff_attr.size) )
        print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.ingress_port_buff_attr.pool_id) )
            
    if attr_item_min.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
        attr_item_min.attr.ingress_port_pg_buff_attr.size = 20
        attr_item_min.attr.ingress_port_pg_buff_attr.pg = 1
        """ is_lossy: 0=Lossless, 1=Lossy """
        attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy = 1
        attr_item_min.attr.ingress_port_pg_buff_attr.xon = 0
        attr_item_min.attr.ingress_port_pg_buff_attr.xoff = 0
        attr_item_min.attr.ingress_port_pg_buff_attr.pool_id = 0
        print ("[%d] Size  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.size) )
        print ("[%d] PG    = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.pg) )
        print ("[%d] is_lossy (0=Lossless)  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy) )
        print ("[%d] Xon  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.xon) )
        print ("[%d] Xoff  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.xoff) )
        print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.pool_id) )

    if attr_item_min.type == SX_COS_EGRESS_PORT_ATTR_E:
        attr_item_min.attr.egress_port_buff_attr.size = 15
        attr_item_min.attr.egress_port_buff_attr.pool_id = 4
        print ("[%d] Size  = %d" % ( i, attr_item_min.attr.egress_port_buff_attr.size) )
        print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.egress_port_buff_attr.pool_id) )
        
    if attr_item_min.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
        attr_item_min.attr.egress_port_tc_buff_attr.size = 15
        attr_item_min.attr.egress_port_tc_buff_attr.tc = 0
        attr_item_min.attr.egress_port_tc_buff_attr.pool_id = 4
        print ("[%d] Size  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.size) )
        print ("[%d] TC  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.tc) )
        print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.pool_id) )
        
    if attr_item_min.type == SX_COS_MULTICAST_ATTR_E:
        attr_item_min.attr.multicast_buff_attr.size = 15
        attr_item_min.attr.multicast_buff_attr.sp = 5
        attr_item_min.attr.multicast_buff_attr.pool_id = 4
        print ("[%d] Size  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.size) )
        print ("[%d] SP  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.sp) )
        print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.pool_id) )
        
    if attr_item_min.type > SX_COS_MULTICAST_ATTR_E or attr_item_min.type < SX_COS_INGRESS_PORT_ATTR_E: 
        print ("[%d] Unsupported union type = %d" % ( i, attr_item_min.type) )

rc = sx_api_cos_port_buff_type_set(handle, cmd, log_port, attr_item_min, port_buffer_attr_cnt)
print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_buffer_attr_cnt, rc) )
if rc != 0:
    sys.exit(errno.EACCES)
    
print ("--------------------------------------------------------------------")


""" PORT TYPE BUFF GET (MIN) """
print ("---------------  PORT TYPE BUFF GET (MIN) ------------------------------")
count = 2
log_port = 0x10100
port_buffer_attr_cnt = new_uint32_t_p()
port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
port_buffer_attr_item = new_sx_cos_port_buffer_attr_t_arr(1)
uint32_t_p_assign(port_buffer_attr_cnt, count)

""" Set type and relevant parameters """
port_buffer_attr_item.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E

""" # ITEM #0 """
if port_buffer_attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
    port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg = 4
    print ("#### [%d] PG    = %d" % ( 0, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg) )

sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, 0, port_buffer_attr_item)

""" Set ITEM#1 """
if port_buffer_attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
    port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg = 1
    print ("#### [%d] PG    = %d" % ( 1, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg) )

sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, 1, port_buffer_attr_item)

rc = sx_api_cos_port_buff_type_get(handle, log_port, port_buffer_attr_list_p, port_buffer_attr_cnt)
if rc != 0:
    sys.exit(errno.EACCES)

attr_cnt = uint32_t_p_value(port_buffer_attr_cnt)
print ("sx_api_cos_port_buff_type_get [log_port=0x%x , cnt=%d, rc=%d] " % ( log_port, attr_cnt, rc) )
print ("--------------------------------------------------------------------")
for i in range(0,attr_cnt):
    port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
    print ("[%d] type = %d" % ( i, port_buffer_attr_item.type) )

    if port_buffer_attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
        print ("[%d] Size  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_buff_attr.size) )
        print ("[%d] pool_id  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_buff_attr.pool_id) )
            
    if port_buffer_attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
        print ("[%d] Size  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.size) )
        print ("[%d] PG    = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pg) )
        print ("[%d] is_lossy (0=Lossless)  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.is_lossy) )
        print ("[%d] Xon  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xon) )
        print ("[%d] Xoff  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.xoff) )
        print ("[%d] pool_id  = %d" % ( i, port_buffer_attr_item.attr.ingress_port_pg_buff_attr.pool_id) )

    if port_buffer_attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
        print ("[%d] Size  = %d" % ( i, port_buffer_attr_item.attr.egress_port_buff_attr.size) )
        print ("[%d] pool_id  = %d" % ( i, port_buffer_attr_item.attr.egress_port_buff_attr.pool_id) )
        
    if port_buffer_attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
        print ("[%d] Size  = %d" % ( i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.size) )
        print ("[%d] TC  = %d" % ( i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.tc) )
        print ("[%d] pool_id  = %d" % ( i, port_buffer_attr_item.attr.egress_port_tc_buff_attr.pool_id) )
        
    if port_buffer_attr_item.type == SX_COS_MULTICAST_ATTR_E:
        port_buffer_attr_item.attr.multicast_buff_attr.size = 15
        port_buffer_attr_item.attr.multicast_buff_attr.sp = 5
        port_buffer_attr_item.attr.multicast_buff_attr.pool_id = 4
        
    if port_buffer_attr_item.type > SX_COS_MULTICAST_ATTR_E or attr_item_min.type < SX_COS_INGRESS_PORT_ATTR_E: 
        print ("[%d] Unsupported union type = %d" % ( i, port_buffer_attr_item.type) )
    


""" ############################################################################################ """
""" PORT SHARED BUFF SET """
print ("---------------  PORT SHARED BUFF SET ------------------------------")
count = 1
log_port = 0x10100
""" CMD_SET = 15 """
cmd = 15
""" CMD_DELETE = 3 """
""" cmd = 3 """
port_shared_buffer_attr_cnt = count
port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)

for i in range(0,port_shared_buffer_attr_cnt):
    attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, i)
    """ Set type and relevant parameters """
    attr_item.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
    
    print ("[%d] type = %d" % ( i, attr_item.type) )
    if attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
        attr_item.attr.ingress_port_shared_buff_attr.max.mode = 1
        print ("[%d] pool_mode  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.mode) )
        if attr_item.attr.ingress_port_shared_buff_attr.max.mode == 0:
            """ Static """
            attr_item.attr.ingress_port_shared_buff_attr.max.max.size = 15
            print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            attr_item.attr.ingress_port_shared_buff_attr.max.max.alpha = 2
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.max.alpha) )
        attr_item.attr.ingress_port_shared_buff_attr.pool_id = 0
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.pool_id) )
            
    if attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
        attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode = 1
        print ("[%d] pool_mode  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode) )
        if attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode == 0:
            """ Static """
            attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.size = 20
            print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha = 3
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha) )
        attr_item.attr.ingress_port_pg_shared_buff_attr.pg = 4
        print ("[%d] PG  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pg) )
        attr_item.attr.ingress_port_pg_shared_buff_attr.pool_id = 0
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pool_id) )

    if attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
        attr_item.attr.egress_port_shared_buff_attr.max.mode = 1
        print ("[%d] pool_mode  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.mode) )
        if attr_item.attr.egress_port_shared_buff_attr.max.mode == 0:
            """ Static """
            attr_item.attr.egress_port_shared_buff_attr.max.max.size = 20
            print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            attr_item.attr.egress_port_shared_buff_attr.max.max.alpha = 4
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.max.alpha) )
          
        attr_item.attr.egress_port_shared_buff_attr.pool_id = 4
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.pool_id) )
    
    if attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
        attr_item.attr.egress_port_tc_shared_buff_attr.max.mode = 1
        print ("[%d] pool_mode  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.mode) )
        if attr_item.attr.egress_port_tc_shared_buff_attr.max.mode == 0:
            """ Static """
            attr_item.attr.egress_port_tc_shared_buff_attr.max.max.size = 30
            print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha = 5
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha) )
          
        attr_item.attr.egress_port_tc_shared_buff_attr.tc = 0
        print ("[%d] TC  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.tc) )
        attr_item.attr.egress_port_tc_shared_buff_attr.pool_id = 4
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.pool_id) )
        
    if attr_item.type == SX_COS_MULTICAST_ATTR_E:
        attr_item.attr.multicast_shared_buff_attr.max.mode = 1
        print ("[%d] pool_mode  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.mode) )
        if attr_item.attr.multicast_shared_buff_attr.max.mode == 0:
            """ Static """
            attr_item.attr.multicast_shared_buff_attr.max.max.size = 50
            print ("[%d] Max  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            attr_item.attr.multicast_shared_buff_attr.max.max.alpha = 6
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.max.alpha) )
          
        attr_item.attr.multicast_shared_buff_attr.sp = 0
        print ("[%d] SP  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.sp) )
        attr_item.attr.multicast_shared_buff_attr.pool_id = 4
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.pool_id) )
 
    if attr_item.type > SX_COS_MULTICAST_ATTR_E or attr_item.type < SX_COS_INGRESS_PORT_ATTR_E: 
        print ("[%d] Unsupported union type = %d" % ( i, attr_item.type) )

rc = sx_api_cos_port_shared_buff_type_set(handle, cmd, log_port, attr_item, port_shared_buffer_attr_cnt)
print ("sx_api_cos_port_shared_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_shared_buffer_attr_cnt, rc) )
if rc != 0:
    sys.exit(errno.EACCES)
    
print ("--------------------------------------------------------------------")


""" PORT SHARED BUFF GET """
print ("---------------  PORT SHARED BUFF GET ------------------------------")
count = 2
log_port = 0x10100

port_shared_buffer_attr_cnt = new_uint32_t_p()
port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
port_shared_buffer_attr_item = new_sx_cos_port_shared_buffer_attr_t_arr(1)
uint32_t_p_assign(port_shared_buffer_attr_cnt, count)

""" Set type and relevant parameters """
port_shared_buffer_attr_item.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E

""" # ITEM #0 """
if port_shared_buffer_attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
    port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.pg = 4
    print ("#### [%d] PG    = %d" % ( 0, port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.pg) )

if port_shared_buffer_attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
    port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.tc = 0
    print ("#### [%d] TC    = %d" % ( 0, port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.tc) )

sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, 0, port_shared_buffer_attr_item)

""" Set ITEM#1 """
if port_shared_buffer_attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
    port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.pg = 0
    print ("#### [%d] PG    = %d" % ( 1, port_shared_buffer_attr_item.attr.ingress_port_pg_shared_buff_attr.pg) )

if port_shared_buffer_attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
    port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.tc = 7
    print ("#### [%d] TC    = %d" % ( 0, port_shared_buffer_attr_item.attr.egress_port_tc_shared_buff_attr.tc) )

sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, 1, port_shared_buffer_attr_item)

rc = sx_api_cos_port_shared_buff_type_get(handle, log_port, port_shared_buffer_attr_list_p, port_shared_buffer_attr_cnt)
if rc != 0:
    sys.exit(errno.EACCES)

attr_cnt = uint32_t_p_value(port_shared_buffer_attr_cnt)
print ("sx_api_cos_port_shared_buff_type_get [log_port=0x%x , cnt=%d, rc=%d] " % ( log_port, attr_cnt, rc) )
print ("--------------------------------------------------------------------")
for i in range(0,attr_cnt):
    attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, i)
    print ("[%d] type = %d" % ( i, attr_item.type) )
    if attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
        if attr_item.attr.ingress_port_shared_buff_attr.max.mode == 0:
            """ Static """
            print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.max.alpha) )

        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.pool_id) )
            
    if attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
        if attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode == 0:
            """ Static """
            print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha) )
          
        print ("[%d] PG  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pg) )
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pool_id) )

    if attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
        if attr_item.attr.egress_port_shared_buff_attr.max.mode == 0:
            """ Static """
            print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.max.alpha) )
          
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.pool_id) )
    if attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
        if attr_item.attr.egress_port_tc_shared_buff_attr.max.mode == 0:
            """ Static """
            print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha) )
          
        print ("[%d] TC  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.tc) )
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.pool_id) )
    if attr_item.type == SX_COS_MULTICAST_ATTR_E:
        if attr_item.attr.multicast_shared_buff_attr.max.mode == 0:
            """ Static """
            print ("[%d] Max  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.max.size) )
        else: 
            """ Dynamic """
            print ("[%d] Max (Alpha) = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.max.alpha) )
          
        print ("[%d] SP  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.sp) )
        print ("[%d] pool_id  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.pool_id) )
    if attr_item.type > SX_COS_MULTICAST_ATTR_E or attr_item.type < SX_COS_INGRESS_PORT_ATTR_E: 
        print ("[%d] Unsupported union type = %d" % ( i, attr_item.type) )

