#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys
import errno, sys
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
from sx_api import *
from test_infra_common import *

port_buffer_type_dict = {0:"INGRESS_PORT", 
	     		 1:"INGRESS_PORT_PRIORITY_GROUP" ,
	     		 2:"EGRESS_PORT" ,
	     		 3:"EGRESS_PORT_TRAFFIC_CLASS" ,
	     		 4:"MULTICAST"}

shared_pool_direction_dict = {0:"INGRESS" ,
			      1:"EGRESS"}

shared_pool_mode_dict = {0:"STATIC" ,
                         1:"DYNAMIC"}

port_shared_buffer_alpha_dict = {0:"SX_COS_PORT_BUFF_ALPHA_0" ,
				 1:"SX_COS_PORT_BUFF_ALPHA_1_128" ,
                                 2:"SX_COS_PORT_BUFF_ALPHA_1_64" ,
                                 3:"SX_COS_PORT_BUFF_ALPHA_1_32" ,
                                 4:"SX_COS_PORT_BUFF_ALPHA_1_16" ,
                                 5:"SX_COS_PORT_BUFF_ALPHA_1_8" ,
                                 6:"SX_COS_PORT_BUFF_ALPHA_1_4" ,
                                 7:"SX_COS_PORT_BUFF_ALPHA_1_2" ,
                                 8:"SX_COS_PORT_BUFF_ALPHA_1" ,
                                 9:"SX_COS_PORT_BUFF_ALPHA_2" ,
                                 10:"SX_COS_PORT_BUFF_ALPHA_4" ,
                                 11:"SX_COS_PORT_BUFF_ALPHA_8" ,
                                 12:"SX_COS_PORT_BUFF_ALPHA_16" ,
                                 13:"SX_COS_PORT_BUFF_ALPHA_32" ,
                                 14:"SX_COS_PORT_BUFF_ALPHA_64" ,
                                 255:"SX_COS_PORT_BUFF_ALPHA_INFINITY"}


def print_all_port_props(log_port):
    port_buffer_attr_cnt = new_uint32_t_p()
    port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(34)
    uint32_t_p_assign(port_buffer_attr_cnt, 34)
    print ("---------------  PORT TYPE BUFF GET (MIN - Reserved Buffers) (Size units = Cell (96 Bytes)) ----")
    rc = sx_api_cos_port_buff_type_get(handle, log_port, port_buffer_attr_list_p, port_buffer_attr_cnt)
    if rc != 0:
        sys.exit(errno.EACCES)
    
    attr_cnt = uint32_t_p_value(port_buffer_attr_cnt)
    print ("sx_api_cos_port_buff_type_get [log_port=0x%x , cnt=%d, rc=%d] " % ( log_port, attr_cnt, rc) )
    print ("------------------------------------------------------------------------------------------------")
    for i in range(0,attr_cnt):
        attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
        print ("[%d] type = %s" % ( i, port_buffer_type_dict[attr_item_min.type]) )
    
        if attr_item_min.type == SX_COS_INGRESS_PORT_ATTR_E:
            print ("[%d] Size  = %d" % ( i, attr_item_min.attr.ingress_port_buff_attr.size) )
            print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.ingress_port_buff_attr.pool_id) )
                
        if attr_item_min.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
            print ("[%d] Size  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.size) )
            print ("[%d] PG    = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.pg) )
            print ("[%d] is_lossy (0=Lossless)  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.is_lossy) )
            print ("[%d] Xon  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.xon) )
            print ("[%d] Xoff  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.xoff) )
            print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.ingress_port_pg_buff_attr.pool_id) )
    
        if attr_item_min.type == SX_COS_EGRESS_PORT_ATTR_E:
            print ("[%d] Size  = %d" % ( i, attr_item_min.attr.egress_port_buff_attr.size) )
            print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.egress_port_buff_attr.pool_id) )
            
        if attr_item_min.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
            print ("[%d] Size  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.size) )
            print ("[%d] TC  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.tc) )
            print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.egress_port_tc_buff_attr.pool_id) )
            
        if attr_item_min.type == SX_COS_MULTICAST_ATTR_E:
            print ("[%d] Size  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.size) )
            print ("[%d] SP  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.sp) )
            print ("[%d] pool_id  = %d" % ( i, attr_item_min.attr.multicast_buff_attr.pool_id) )
            
        if attr_item_min.type > SX_COS_MULTICAST_ATTR_E or attr_item_min.type < SX_COS_INGRESS_PORT_ATTR_E: 
            print ("[%d] Unsupported union type = %d" % ( i, attr_item_min.type) )

    """ PORT SHARED BUFF GET """
    print ("---------------  PORT SHARED BUFF GET (MAX - Shared Buffers) (Size units = Cell (96 Bytes)) ----")
    port_shared_buffer_attr_cnt = new_uint32_t_p()
    port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(34)
    uint32_t_p_assign(port_shared_buffer_attr_cnt, 34)
    
    rc = sx_api_cos_port_shared_buff_type_get(handle, log_port, port_shared_buffer_attr_list_p, port_shared_buffer_attr_cnt)
    if rc != 0:
        sys.exit(errno.EACCES)
    
    attr_cnt = uint32_t_p_value(port_shared_buffer_attr_cnt)
    print ("sx_api_cos_port_shared_buff_type_get [log_port=0x%x , cnt=%d, rc=%d] " % ( log_port, attr_cnt, rc) )
    print ("------------------------------------------------------------------------------------------------")
    for i in range(0,attr_cnt):
        attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, i)
        print ("[%d] type = %s" % ( i, port_buffer_type_dict[attr_item.type]) )
        if attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
            if attr_item.attr.ingress_port_shared_buff_attr.max.mode == 0:
                """ Static """
                print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.max.max.size) )
            else: 
                """ Dynamic """
                print ("[%d] Max (Alpha) = %s" % ( i, port_shared_buffer_alpha_dict[attr_item.attr.ingress_port_shared_buff_attr.max.max.alpha]) )
    
            print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_shared_buff_attr.pool_id) )
                
        if attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
            if attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode == 0:
                """ Static """
                print ("[%d] Max  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.size) )
            else: 
                """ Dynamic """
                print ("[%d] Max (Alpha) = %s" % ( i, port_shared_buffer_alpha_dict[attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.alpha]) )
              
            print ("[%d] PG  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pg) )
            print ("[%d] pool_id  = %d" % ( i, attr_item.attr.ingress_port_pg_shared_buff_attr.pool_id) )
    
        if attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
            if attr_item.attr.egress_port_shared_buff_attr.max.mode == 0:
                """ Static """
                print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.max.max.size) )
            else: 
                """ Dynamic """
                print ("[%d] Max (Alpha) = %s" % ( i, port_shared_buffer_alpha_dict[attr_item.attr.egress_port_shared_buff_attr.max.max.alpha]) )
              
            print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_shared_buff_attr.pool_id) )
        if attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
            if attr_item.attr.egress_port_tc_shared_buff_attr.max.mode == 0:
                """ Static """
                print ("[%d] Max  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.max.max.size) )
            else: 
                """ Dynamic """
                print ("[%d] Max (Alpha) = %s" % ( i, port_shared_buffer_alpha_dict[attr_item.attr.egress_port_tc_shared_buff_attr.max.max.alpha]) )
              
            print ("[%d] TC  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.tc) )
            print ("[%d] pool_id  = %d" % ( i, attr_item.attr.egress_port_tc_shared_buff_attr.pool_id) )
        if attr_item.type == SX_COS_MULTICAST_ATTR_E:
            if attr_item.attr.multicast_shared_buff_attr.max.mode == 0:
                """ Static """
                print ("[%d] Max  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.max.max.size) )
            else: 
                """ Dynamic """
                print ("[%d] Max (Alpha) = %s" % ( i, port_shared_buffer_alpha_dict[attr_item.attr.multicast_shared_buff_attr.max.max.alpha]) )
              
            print ("[%d] SP  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.sp) )
            print ("[%d] pool_id  = %d" % ( i, attr_item.attr.multicast_shared_buff_attr.pool_id) )
        if attr_item.type > SX_COS_MULTICAST_ATTR_E or attr_item.type < SX_COS_INGRESS_PORT_ATTR_E: 
            print ("[%d] Unsupported union type = %d" % ( i, attr_item.type) )


def print_all_shared_pool_props():
    shared_pool_id = new_uint32_t_p()
    shared_pool_attr_p = new_sx_cos_pool_attr_t_arr(1)
    uint32_t_p_assign(shared_pool_id, 8)
    print ("---------------  SHARED POOL GET (Size units = Cell (96 Bytes)) --------------------------------")
    pool_id_cnt = uint32_t_p_value(shared_pool_id)

    for shared_pool_id in range(0,pool_id_cnt):
        rc = sx_api_cos_shared_buff_pool_get(handle, shared_pool_id, shared_pool_attr_p)
        if rc != 0:
            sys.exit(errno.EACCES)

    	print ("sx_api_cos_shared_buff_pool_get [pool_id=%d , rc=%d] " % ( shared_pool_id, rc) )
	print ("================================================================================================")

        pool_attr_item = sx_cos_pool_attr_t_arr_getitem(shared_pool_attr_p, 0)
        print ("[Pool ID: %d] Pool direction = %s" % ( shared_pool_id, shared_pool_direction_dict[pool_attr_item.pool_dir]) )
        print ("[Pool ID: %d] Pool size  = %d" % ( shared_pool_id, pool_attr_item.pool_size) )
        print ("[Pool ID: %d] Pool mode  = %s" % ( shared_pool_id, shared_pool_mode_dict[pool_attr_item.mode]) )
	print ("\n")


def delete_all_mc_sb():
	print ("--  Delete all MC --")
	count = 1
	log_port = 0x40000000
	""" CMD_DELETE = 3 """
	cmd = 3 
	port_buffer_attr_cnt = count
	port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
	
	for i in range(0,port_buffer_attr_cnt):
	    attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
	    """ Set type and relevant parameters """
	    attr_item_min.type = SX_COS_MULTICAST_ATTR_E
	    if attr_item_min.type > SX_COS_MULTICAST_ATTR_E or attr_item_min.type < SX_COS_INGRESS_PORT_ATTR_E: 
	        print ("[%d] Unsupported union type = %d" % ( i, attr_item_min.type) )
	
	rc = sx_api_cos_port_buff_type_set(handle, cmd, log_port, attr_item_min, port_buffer_attr_cnt)
	print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_buffer_attr_cnt, rc) )
	if rc != 0:
	    sys.exit(errno.EACCES)
	
	port_shared_buffer_attr_cnt = count
	port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
	
	for i in range(0,port_shared_buffer_attr_cnt):
	    attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, i)
	    """ Set type and relevant parameters """
	    attr_item.type = SX_COS_MULTICAST_ATTR_E
	    if attr_item.type > SX_COS_MULTICAST_ATTR_E or attr_item.type < SX_COS_INGRESS_PORT_ATTR_E: 
	        print ("[%d] Unsupported union type = %d" % ( i, attr_item.type) )
	
	rc = sx_api_cos_port_shared_buff_type_set(handle, cmd, log_port, attr_item, port_shared_buffer_attr_cnt)
	print ("sx_api_cos_port_shared_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_shared_buffer_attr_cnt, rc) )
	if rc != 0:
	    sys.exit(errno.EACCES)
	    
	print ("--  Delete all MC - DONE --")

def delete_all_port_buff_sb(log_port):
	print ("--  Delete Port buff SB --")
	count = 1
	""" CMD_DELETE = 3 """
	cmd = 3 
	port_buffer_attr_cnt = count
	port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
	
	for i in range(0,port_buffer_attr_cnt):
	    attr_item_min = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
	
	rc = sx_api_cos_port_buff_type_set(handle, cmd, log_port, attr_item_min, port_buffer_attr_cnt)
	print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_buffer_attr_cnt, rc) )
	if rc != 0:
	    sys.exit(errno.EACCES)
	
	port_shared_buffer_attr_cnt = count
	port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
	
	for i in range(0,port_shared_buffer_attr_cnt):
	    attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, i)
	
	rc = sx_api_cos_port_shared_buff_type_set(handle, cmd, log_port, attr_item, port_shared_buffer_attr_cnt)
	print ("sx_api_cos_port_shared_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % ( cmd, log_port, port_shared_buffer_attr_cnt, rc) )
	if rc != 0:
	    sys.exit(errno.EACCES)
	    
	print ("--  Delete Port buff SB - DONE--")

def delete_all_pools_buff_sb():
	print ("--  Delete all Pools buff SB --")
	shared_pool_id = new_uint32_t_p()
	shared_pool_attr_p = new_sx_cos_pool_attr_t_arr(1)
	uint32_t_p_assign(shared_pool_id, 7)
	pool_id_cnt = uint32_t_p_value(shared_pool_id)
	for pool_index in range(0,pool_id_cnt):
		uint32_t_p_assign(shared_pool_id, pool_index)
		rc = sx_api_cos_shared_buff_pool_set(handle, SX_ACCESS_CMD_DESTROY, shared_pool_attr_p, shared_pool_id)
        if rc != 0:
            sys.exit(errno.EACCES)

    	print ("sx_api_cos_shared_buff_pool_set [pool_id=%d , rc=%d] " % ( shared_pool_id, rc) )
	print ("--  Delete all Pools buff SB - DONE--")
	print ("\n")

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )


port_cnt_p = new_uint32_t_p()
port_attributes_list = new_sx_port_attributes_t_arr(64)
port_attributes = sx_port_attributes_t()
uint32_t_p_assign(port_cnt_p,64)
rc = sx_api_port_device_get(handle, 1 , 0, port_attributes_list,  port_cnt_p)
port_cnt = uint32_t_p_value(port_cnt_p)
print ("sx_api_port_device_get port_cnt:%d , rc %d, port_list : " % (port_cnt, rc) )
print ("====================================================================================================")
for i in range(0,port_cnt):
	port_attributes = sx_port_attributes_t_arr_getitem(port_attributes_list,i)
	is_nve = check_nve(int(port_attributes.log_port))
	if is_nve:
		continue
	delete_all_port_buff_sb(port_attributes.log_port)

delete_all_mc_sb()

delete_all_pools_buff_sb()
