#!/usr/bin/env python
'''
This file contains Python command example for the CoS module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different CoS attributes.
This example is supported on Spectrum devices. 
'''
import sys
import errno, sys
import colorsys
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
from sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)



""" ############################################################################################ """

#sx_api_cos_port_dscp_to_prio_set  0x12300 [0,1,2,3,4,5,6,7] 
#[{7;0},{6;0},{5;0},{4;0},{3;0},{2;0},{1;0},{0;0}] 8
""" COS PORT PCP DSCP TO PRIO SET """
print ("--------------- COS PORT DSCP TO PRIO SET ------------------------------")

log_port = 0x10100
element_count = 8
dscp_p = new_sx_cos_dscp_t_arr(element_count)
switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)

for i in range(0,element_count):
    switch_priority_color = sx_cos_priority_color_t()
        
    dscp = i%64
    switch_priority_color.priority= i%15
    switch_priority_color.color= i%3
   
    sx_cos_dscp_t_arr_setitem(dscp_p, i, dscp)
    sx_cos_priority_color_t_arr_setitem(switch_priority_color_p, i,switch_priority_color)


rc = sx_api_cos_port_dscp_to_prio_set(handle, log_port, dscp_p, switch_priority_color_p, element_count)
print ("sx_api_cos_port_dscp_to_prio_set for log_port=0x%x, rc=%d" % (log_port, rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)



""" ############################################################################################ """

#sx_api_cos_port_dscp_to_prio_get  0x10100 8
""" COS PORT DSCP TO PRIO GET """
print ("--------------- COS PORT DSCP TO PRIO GET ------------------------------")

log_port = 0x10100
element_count_p = new_uint32_t_p()
""" Set cnt = 8 """
uint32_t_p_assign(element_count_p, 8) 
""" Get value from PTR """    
element_count = uint32_t_p_value(element_count_p)
print ("element_count (returned from API = %d)" % (element_count)) 

dscp_p = new_sx_cos_dscp_t_arr(element_count)
switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)

rc = sx_api_cos_port_dscp_to_prio_get(handle,log_port,dscp_p,switch_priority_color_p,element_count_p)
print ("sx_api_cos_port_dscp_to_prio_get for log_port=0x%x, rc=%d " % (log_port, rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
 


for i in range(0,element_count):
    dscp = sx_cos_dscp_t_arr_getitem(dscp_p, i)   
    print("dscp=%d " %(dscp))
    
    switch_priority_color = sx_cos_priority_color_t_arr_getitem(switch_priority_color_p, i)
    print("priority=%d, color=%d " % (switch_priority_color.priority, switch_priority_color.color))
    
""" ############################################################################################ """

""" COS PORT TRUST SET """
print ("--------------- COS PORT TRUST SET ------------------------------")

log_port = 0x10100
trust_state = SX_COS_TRUST_LEVEL_L3

rc =  sx_api_cos_port_trust_set(handle, log_port, trust_state)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print ("sx_api_cos_port_trust_set [log_port=0x%x , trust_state=%d] " % ( log_port, trust_state) )

""" ############################################################################################ """

""" COS PORT TRUST GET"""
print ("--------------- COS PORT TRUST GET ------------------------------")

log_port = 0x10100
trust_state_p = new_sx_cos_trust_level_t_p()

rc = sx_api_cos_port_trust_get(handle, log_port, trust_state_p)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
trust_state = sx_cos_trust_level_t_p_value(trust_state_p)
print ("--------------------------------------------------------------------")
print ("sx_api_cos_port_trust_get [log_port=0x%x , trust_state=%d] " % ( log_port, trust_state) )

""" ############################################################################################ """

# 0xsx_api_cos_port_rewrite_enable_set 10100 {1;1;1}
""" COS PORT REWRITE ENABLE SET """
print ("--------------- COS PORT REWRITE ENABLE SET ------------------------------")

log_port = 0x10100
rewrite = sx_cos_rewrite_enable_t()
rewrite.rewrite_pcp_dei = 1
rewrite.rewrite_dscp = 1
rewrite.rewrite_exp = 1 

rc = sx_api_cos_port_rewrite_enable_set(handle, log_port, rewrite)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print ("sx_api_cos_port_rewrite_enable_set (1,1,1) for log_port=0x%x " % (log_port) )

""" ############################################################################################ """
#sx_api_cos_port_rewrite_enable_get 0x10100 
""" COS PORT REWRITE ENABLE GET """
print ("--------------- COS PORT REWRITE ENABLE GET ------------------------------")

rewrite_enable_p = new_sx_cos_rewrite_enable_t_p()

rc =  sx_api_cos_port_rewrite_enable_get(handle, log_port, rewrite_enable_p)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
rewrite_p = sx_cos_rewrite_enable_t_p_value(rewrite_enable_p)

print ("sx_api_cos_port_rewrite_enable_get (pcp_dei=%d, dscp=%d, exp=%d) for log_port=0x%x ] " % ( rewrite_p.rewrite_pcp_dei, rewrite_p.rewrite_dscp, rewrite_p.rewrite_exp, log_port) )

""" ############################################################################################ """

#sx_api_cos_port_prio_to_pcpdei_rewrite_set  0x12300 [{0;0},{1;1}] [{0;0},{1;1}] 2
""" COS PORT PRIO TO PCP DEI REWRITE SET """
print ("--------------- COS PORT PRIO TO PCP DEI REWRITE ENABLE SET ------------------------------")

log_port = 0x12300
element_count = 2 
switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)
pcp_dei_p = new_sx_cos_pcp_dei_t_arr(element_count)


for i in range(0,element_count):
    switch_priority_color = sx_cos_priority_color_t()
    pcp_dei = sx_cos_pcp_dei_t()
    
    switch_priority_color.priority= i%15
    switch_priority_color.color= i%3
    pcp_dei.pcp = i%8
    pcp_dei.dei = i%2
   
    sx_cos_priority_color_t_arr_setitem(switch_priority_color_p, i,switch_priority_color)
    sx_cos_pcp_dei_t_arr_setitem(pcp_dei_p, i, pcp_dei)

rc = sx_api_cos_port_prio_to_pcpdei_rewrite_set(handle, log_port, switch_priority_color_p, pcp_dei_p, element_count)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print ("sx_api_cos_port_prio_to_pcpdei_rewrite_set for log_port=0x%x " % (log_port) )


""" ############################################################################################ """

#sx_api_cos_port_prio_to_pcpdei_rewrite_get  0x12300 2
""" COS PORT PRIO TO PCP DEI REWRITE GET """
print ("--------------- COS PORT PRIO TO PCP DEI REWRITE ENABLE GET ------------------------------")

log_port = 0x12300
element_count_p = new_uint32_t_p()
""" Set cnt = 2 """
uint32_t_p_assign(element_count_p, 2) 
""" Get value from PTR """    
element_count = uint32_t_p_value(element_count_p)
print ("element_count (returned from API = %d)" % (element_count)) 

switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)
pcp_dei_p = new_sx_cos_pcp_dei_t_arr(element_count)

rc = sx_api_cos_port_prio_to_pcpdei_rewrite_get(handle,log_port,switch_priority_color_p, pcp_dei_p, element_count_p)
print ("sx_api_cos_port_prio_to_pcpdei_rewrite_get for log_port=0x%x, rc=%d " % (log_port, rc) ) 
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)


for i in range(0,element_count):
    switch_priority_color = sx_cos_priority_color_t_arr_getitem(switch_priority_color_p, i)
    print("priority=%d, color=%d " % (switch_priority_color.priority, switch_priority_color.color))
    
    pcp_dei = sx_cos_pcp_dei_t_arr_getitem(pcp_dei_p, i)   
    print("pcp=%d, dei=%d " %(pcp_dei.pcp, pcp_dei.dei))
    

""" ############################################################################################ """
# sx_api_cos_port_prio_to_dscp_rewrite_set  0x12300 [{0;0}, {1;1}] [0,1] 2
""" COS PORT PRIO TO DSCP REWRITE SET """
print ("--------------- COS PORT PRIO TO DSCP REWRITE ENABLE SET ------------------------------")

log_port = 0x12300
element_count = 2
switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)
dscp_p = new_sx_cos_dscp_t_arr(element_count)


for i in range(0,element_count):
    switch_priority_color = sx_cos_priority_color_t()
    pcp_dei = sx_cos_pcp_dei_t()
    switch_priority_color.priority= i%15
    switch_priority_color.color= i%3
    dscp = i%64
   
    sx_cos_priority_color_t_arr_setitem(switch_priority_color_p, i,switch_priority_color)
    sx_cos_dscp_t_arr_setitem(dscp_p, i, dscp)

rc = sx_api_cos_port_prio_to_dscp_rewrite_set(handle, log_port, switch_priority_color_p, dscp_p, element_count)
print ("sx_api_cos_port_prio_to_dscp_rewrite_set for log_port=0x%x, rc =%d" % (log_port, rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)


""" ############################################################################################ """

#sx_api_cos_port_prio_to_dscp_rewrite_get  0x12300 2
""" COS PORT PRIO TO DSCP REWRITE GET """
print ("--------------- COS PORT PRIO TO DSCP REWRITE ENABLE GET ------------------------------")

log_port = 0x12300
element_count_p = new_uint32_t_p()
""" Set cnt = 2 """
uint32_t_p_assign(element_count_p, 2) 
""" Get value from PTR """    
element_count = uint32_t_p_value(element_count_p)
print ("element_count (returned from API = %d)" % (element_count)) 

switch_priority_color_p = new_sx_cos_priority_color_t_arr(element_count)
dscp_p = new_sx_cos_dscp_t_arr(element_count)

rc = sx_api_cos_port_prio_to_dscp_rewrite_get(handle,log_port,switch_priority_color_p, dscp_p, element_count_p)
print ("sx_api_cos_port_prio_to_dscp_rewrite_get for log_port=0x%x, rc=%d " % (log_port, rc) ) 
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)


for i in range(0,element_count):
    switch_priority_color = sx_cos_priority_color_t_arr_getitem(switch_priority_color_p, i)
    print("priority=%d, color=%d " % (switch_priority_color.priority, switch_priority_color.color))
    
    dscp = sx_cos_dscp_t_arr_getitem(dscp_p, i)   
    print("dscp=%d " %(dscp))
    

""" ############################################################################################ """
#sx_api_cos_prio_to_ieeeprio_set [0,1,2,3,4,5,6,7,8,9] [[0,1,2,3,4,5,6,7,0,1] 10
""" COS PRIO TO IEEE PRIORITY SET """
print ("--------------- COS PRIO TO IEEE PRIORITY SET ------------------------------")

element_count = 10
switch_prio_p = new_sx_cos_priority_t_arr(element_count)
ieee_prio_p = new_sx_cos_ieee_prio_t_arr(element_count)


for i in range(0,element_count):
    ieee_priority = i%8
    priority= i
   
    sx_cos_priority_t_arr_setitem(switch_prio_p, i, priority)
    sx_cos_ieee_prio_t_arr_setitem(ieee_prio_p, i, ieee_priority)
    
    switch_prio = sx_cos_priority_t_arr_getitem(switch_prio_p, i)
    ieee_prio = sx_cos_ieee_prio_t_arr_getitem(ieee_prio_p, i)
    print("Setter switch priority=%d, ieee priority=%d " % (switch_prio,ieee_prio))
    
rc = sx_api_cos_prio_to_ieeeprio_set(handle, switch_prio_p, ieee_prio_p, element_count)
print ("sx_api_cos_prio_to_ieeeprio_set- rc=%d" % (rc) )
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)

""" ############################################################################################ """
#sx_api_cos_prio_to_ieeeprio_get 10
""" COS PRIO TO IEEE PRIORITY GET """
print ("--------------- COS PRIO TO IEEE PRIORITY GET ------------------------------")

element_count_p = new_uint32_t_p()
""" Set cnt = 10 """
uint32_t_p_assign(element_count_p, 10) 
""" Get value from PTR """    
element_count = uint32_t_p_value(element_count_p)
print ("element_count (returned from API = %d)" % (element_count)) 

switch_prio_p = new_sx_cos_priority_t_arr(element_count)
ieee_prio_p = new_sx_cos_ieee_prio_t_arr(element_count)

rc = sx_api_cos_prio_to_ieeeprio_get(handle, switch_prio_p, ieee_prio_p, element_count_p)
print ("sx_api_cos_prio_to_ieeeprio_get, rc=%d " % (rc) ) 
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)

for i in range(0,element_count):
    switch_prio = sx_cos_priority_t_arr_getitem(switch_prio_p, i)
    ieee_prio = sx_cos_ieee_prio_t_arr_getitem(ieee_prio_p, i)
    print("switch priority=%d, ieee priority=%d " % (switch_prio,ieee_prio))

""" ############################################################################################ """


""" COS PORT TC PRIO MAP SET  AND GET """
print ("--------------- COS PORT TC PRIO MAP SET AND GET ------------------------------")

element_count_p = new_uint32_t_p()
""" Set cnt = 8 """
uint32_t_p_assign(element_count_p, 8) 
""" Get value from PTR """    
element_count = uint32_t_p_value(element_count_p)
print ("element_count (returned from API = %d)" % (element_count)) 

log_port = 0x10100
for i in range(0,element_count):
    priority = i
    traffic_class = i
    cmd = SX_ACCESS_CMD_ADD
    
    rc = sx_api_cos_port_tc_prio_map_set(handle, cmd, log_port, priority, traffic_class)
    print("rc = %d" %(rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("sx_api_cos_port_tc_prio_map_set [log_port=0x%x , prio=%d, tc=%d] " % ( log_port, priority, traffic_class) )
    
    
    traffic_class_p = new_sx_cos_traffic_class_t_p()
    rc = sx_api_cos_port_tc_prio_map_get(handle, log_port, priority, traffic_class_p)
    print("rc = %d" % (rc))
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    traffic_class = sx_cos_traffic_class_t_p_value(traffic_class_p)
    print ("sx_api_cos_port_tc_prio_map_get [log_port=0x%x , prio=%d, tc=%d] " % ( log_port, priority, traffic_class) )

""" ############################################################################################ """

""" COS PORT TC MCAWARE SET """
print ("--------------- COS PORT TC MCAWARE SET ------------------------------")

log_port = 0x10100
mc_aware = False

rc = sx_api_cos_port_tc_mcaware_set(handle, log_port, mc_aware)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print ("sx_api_cos_port_tc_mcaware_set [log_port=0x%x , mc_aware=%d] " % ( log_port, mc_aware) )

""" ############################################################################################ """

""" COS PORT TC MCAWARE GET """
print ("--------------- COS PORT TC MCAWARE GET ------------------------------")

log_port = 0x10100
mc_aware_p = new_boolean_t_p()

rc = sx_api_cos_port_tc_mcaware_get(handle, log_port, mc_aware_p)
print("rc = %d" %(rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
mc_aware = boolean_t_p_value(mc_aware_p)
print ("sx_api_cos_port_tc_mcaware_get [log_port=0x%x , mc_aware=%d] " % ( log_port, mc_aware) )

""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
