#!/usr/bin/env python
'''
This file contains Python command example for the FDB module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

\section cases Example Flow

The example below performs the following configuration operations:

    1. Creates VxLAN tunnel.

    2. Creates FDB entry.

    3. Prints FDB table.

'''
import os
import sys
import errno, sys ,struct ,socket
import colorsys\


sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')

from sx_api import *
import sx_api_router
from optparse import OptionParser,Option

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)
        
###################################################### 
#    defines
######################################################
SPECTRUM_SWID = 0
VLAN_NVE = 5
PORT1 = 0x11700
PORT_NVE = 0x80010000

###################################################### 
#    Functions API
######################################################
'''
This part contains all the calls to that sdk API functions
'''
""" ############################################################################################ """

entry_dict = {0 : 'STATIC', 1 : 'REMOTE', 2 : 'NONAGEABLE', 3 : 'AGEABLE', 4 : 'ALL', \
               5 : 'AGED', 6 : 'NONAGED', 7 : 'POLICY_INVALID'}
action_dict = {0 : 'FORWARD', 1 : 'MIRROR_TO_CPU', 2 : 'TRAP', 3 : 'FORWARD_TO_ROUTER', \
               4 : 'INVALID', 5 : 'INVALID', 6 : 'INVALID', 7 : 'INVALID', 8 : 'INVALID', 9 : 'INVALID', 10 : 'INVALID', 11 : 'INVALID', \
               12 : 'INVALID', 13 : 'INVALID', 14 : 'INVALID', 15 : 'DISCARD'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

# Open API
rc, handle2 = sx_api_open(None)
print("handle: 0x%x , rc: %d " %(handle, rc))

def print_all_attr_from_mac_entry(mac_entry):
    for attr in dir(mac_entry):
        if not attr.startswith("__") and not attr == "this":
            if attr == "mac_addr":
                print "\tmac_addr = %s" % (mac_entry.mac_addr.to_str())
            elif attr == "log_port":
                print "\tlog_port = 0x%x" % (getattr(mac_entry, attr))
            elif attr == "entry_type":
                print "\tentry_type = %s" % entry_dict[(getattr(mac_entry, attr))]
            elif attr == "action":
                print "\taction = %s" % action_dict[(getattr(mac_entry, attr))]
            else:
                print "\t%s = %s" % (attr, getattr(mac_entry, attr))

""" ############################################################################################ """

def tunnel_init(tunnel_general_params):
    
    rc = sx_api_tunnel_init_set(handle,tunnel_general_params)
    print ("sx_api_tunnel_init_set  [rc = %d] " % (rc) )
    assert SX_STATUS_SUCCESS == rc, "Failed to init tunnel, rc: %d" % (rc)

def router_init():
        " This function init the router with following values. "

        general_params = sx_router_general_param_t()
        general_params.ipv4_enable    = SX_ROUTER_ENABLE_STATE_ENABLE
        general_params.ipv4_mc_enable = SX_ROUTER_ENABLE_STATE_ENABLE
        general_params.ipv4_mc_enable = SX_ROUTER_ENABLE_STATE_DISABLE
        general_params.ipv6_mc_enable = SX_ROUTER_ENABLE_STATE_DISABLE
        general_params.rpf_enable     = SX_ROUTER_ENABLE_STATE_DISABLE
        router_resource = sx_router_resources_param_t()
        router_resource.max_virtual_routers_num     = 30
        router_resource.max_vlan_router_interfaces  = 30
        router_resource.max_port_router_interfaces  = 30
        router_resource.max_router_interfaces       = 30
        router_resource.min_ipv4_neighbor_entries   = 10
        router_resource.min_ipv6_neighbor_entries   = 10
        router_resource.min_ipv4_uc_route_entries   = 10
        router_resource.min_ipv6_uc_route_entries   = 10
        router_resource.min_ipv4_mc_route_entries   = 0
        router_resource.min_ipv6_mc_route_entries   = 0
        router_resource.max_ipv4_neighbor_entries   = 1000
        router_resource.max_ipv6_neighbor_entries   = 1000
        router_resource.max_ipv4_uc_route_entries   = 1000
        router_resource.max_ipv6_uc_route_entries   = 1000
        router_resource.max_ipv4_mc_route_entries   = 0
        router_resource.max_ipv6_mc_route_entries   = 0

        rc = sx_api_router_init_set(handle,general_params,router_resource)
        assert SX_STATUS_SUCCESS == rc or SX_STATUS_ALREADY_INITIALIZED, "Failed to init the router"

        print "Init the router, rc: %d" % (rc)

def create_vrid():
    " This function creates vrid. "

    router_attr = sx_router_attributes_t()
    router_attr.ipv4_enable             = SX_ROUTER_ENABLE_STATE_ENABLE
    router_attr.ipv6_enable             = SX_ROUTER_ENABLE_STATE_DISABLE
    router_attr.ipv4_mc_enable          = SX_ROUTER_ENABLE_STATE_DISABLE
    router_attr.ipv6_mc_enable          = SX_ROUTER_ENABLE_STATE_DISABLE
    router_attr.uc_default_rule_action  = SX_ROUTER_ACTION_DROP
    router_attr.mc_default_rule_action  = SX_ROUTER_ACTION_DROP

    vrid_p = new_sx_router_id_t_p()
    rc = sx_api_router_set(handle, SX_ACCESS_CMD_ADD, router_attr, vrid_p)
    assert SX_STATUS_SUCCESS == rc, "Failed to create VRID"

    vrid = sx_router_id_t_p_value(vrid_p)
    print "Created VRID: %d, rc: %d " % (vrid, rc)

    return vrid


def delete_vrid(vrid):
    " This function deletes vrid. "

    vrid_p = new_sx_router_id_t_p()
    sx_router_id_t_p_assign(vrid_p, vrid)
    rc = sx_api_router_set(handle, SX_ACCESS_CMD_DELETE, None, vrid_p)
    assert SX_STATUS_SUCCESS == rc, "Failed to delete VRID %d" % (vrid)
    print("Deleted VRID: %d, rc: %d " % (vrid, rc))

def vport_add_delete(cmd, log_port, vid, log_vport_p):

    """ VIRTUAL PORT CREATE AND DELETE """

    print("log port =0x%x " % (log_port))
    print("vlan id=%d " % (vid))

    rc = sx_api_port_vport_set(handle, cmd, log_port, vid, log_vport_p)
    print ("sx_api_port_vport_set [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

""" ############################################################################################ """

def bridge_create_delete(cmd, bridge_id_p):
    """ BRIDGE CREATE/DELETE"""

    rc = sx_api_bridge_set(handle, cmd, bridge_id_p)
    print ("sx_api_bridge_set  [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

def bridge_vport_add_delete(cmd, bridge_id, log_port):
    """ ADD/DELETE VPORT TO/FROM BRIDGE """

    rc = sx_api_bridge_vport_set(handle, cmd, bridge_id, log_port)
    print ("sx_api_bridge_set  [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

def port_state_set(log_port, admin_state):

    """ PORT STATE SET """

    rc = sx_api_port_state_set(handle, log_port, admin_state)
    print ("sx_api_port_state_set  [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)
    print ("log port = 0x%x, admin state = %d"  % ( log_port, admin_state) )


def vlan_ports_add_delete(cmd, vid, vlan_port_list_p, port_cnt):
    """ ADD/DELETE VLAN MEMBER PORTS """

    rc = sx_api_vlan_ports_set(handle, cmd, SPECTRUM_SWID, vid, vlan_port_list_p, port_cnt)
    print ("sx_api_vlan_ports_set  [rc=%d] " % ( rc) )
    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)

def allocate_vlan_port_list(ports_dict):
    " This function allocates sx_vlan_ports_t array, out of port dictionary. "

    port_list = new_sx_vlan_ports_t_arr(len(ports_dict))
    for i, port_id in enumerate(ports_dict.keys()):
        vlan_port = sx_vlan_ports_t()
        vlan_port.log_port = port_id
        vlan_port.is_untagged = ports_dict[port_id]
        sx_vlan_ports_t_arr_setitem(port_list, i, vlan_port)
    return port_list

def add_ports_to_vlan(vlan_id, ports_dict):
    " This function adds speificed ports_dict into target vlan. "

    port_list = allocate_vlan_port_list(ports_dict)
    rc = sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_ADD, SPECTRUM_SWID, vlan_id, port_list, len(ports_dict))
    assert SX_STATUS_SUCCESS == rc, "Failed to add port %s to vlan %d" % (str(ports_dict.keys()), vlan_id)
    for port in ports_dict:
        print "Added port 0x%x to vlan %d, rc: %d" % (port, vlan_id, rc)

def remove_ports_from_vlan(vlan_id, ports_dict):
    " This function removes ports from given vlan. "

    port_list = allocate_vlan_port_list(ports_dict)
    rc = sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_DELETE, SPECTRUM_SWID, vlan_id, port_list, len(ports_dict));
    assert SX_STATUS_SUCCESS == rc, "Failed to remove ports %s from vlan %d" % (str(ports_dict.keys()), vlan_id)
    for port in ports_dict:
        print "Removed port 0x%x from vlan %d, rc: %d" % (port, vlan_id, rc)

def tunnel_map_entry_add(tunnel_id,bridge_id):
    map_entry = make_tunnel_map_entry_params(bridge_id)
    map_entry_p = new_sx_tunnel_map_entry_t_p()
    sx_tunnel_map_entry_t_p_assign(map_entry_p,map_entry)
    return sx_api_tunnel_map_set(handle,SX_ACCESS_CMD_ADD,tunnel_id,map_entry_p,1)

def tunnel_map_entry_delete(tunnel_id,bridge_id):
    map_entry = make_tunnel_map_entry_params(bridge_id)
    map_entry_p = new_sx_tunnel_map_entry_t_p()
    sx_tunnel_map_entry_t_p_assign(map_entry_p,map_entry)
    return sx_api_tunnel_map_set(handle,SX_ACCESS_CMD_DELETE_ALL,tunnel_id,map_entry_p,1)

def make_tunnel_map_entry_params(bridge_id):
    map_entry = sx_tunnel_map_entry_t()
    map_entry.type = SX_TUNNEL_TYPE_NVE_VXLAN
    map_entry.params.nve.bridge_id = bridge_id
    map_entry.params.nve.vni = 5

    return map_entry

def prepare_nve_bridge(tunnel_id, vlan):
    bridge_id, log_vport_1 = bridge_vport_create_example(vlan, PORT1)
    rc = tunnel_map_entry_add(tunnel_id, bridge_id)

    return bridge_id, log_vport_1

""" ############################################################################################ """

def tunnel_deinit():
    rc = sx_api_tunnel_deinit_set(handle)
    print ("sx_api_tunnel_deinit_set  [rc = %d] " % (rc) )
    assert SX_STATUS_SUCCESS == rc, "Failed to deinit tunnel, rc: %d" % (rc)

""" ############################################################################################ """

def tunnel_create(tunnel_attribute ):
    tunnel_id_p = new_sx_tunnel_id_t_p()
    rc = sx_api_tunnel_set(handle ,SX_ACCESS_CMD_CREATE ,tunnel_attribute , tunnel_id_p) 
    print ("sx_api_tunnel_set  [rc = %d] " % (rc) )
    assert SX_STATUS_SUCCESS == rc, "Failed to create tunnel, rc: %d" % (rc)

    tunnel_id = sx_tunnel_id_t_p_value(tunnel_id_p)
    return tunnel_id

""" ############################################################################################ """

def tunnel_destroy(tunnel_id_p, delete_tunnel_after_entry = True):
    tunnel_attribute = sx_tunnel_attribute_t()
    rc = sx_api_tunnel_set(handle , SX_ACCESS_CMD_DESTROY ,tunnel_attribute , tunnel_id_p) 
    print ("sx_api_tunnel_set  [rc = %d] " % (rc) )
    if delete_tunnel_after_entry == False:
        assert SX_STATUS_RESOURCE_IN_USE == rc, "unnel destroy should fail, rc: %d" % (rc)
    else:
        assert SX_STATUS_SUCCESS == rc, "Tunnel destroy should not fail, rc: %d" % (rc)

""" ############################################################################################ """

def create_loopback_rif(vrid, vlan, mtu=1500): 
    " This function creates loopback rif with given parameters. "
    
    ifc_param = sx_router_interface_param_t()
    ifc_param.type = SX_L2_INTERFACE_TYPE_LOOPBACK
    ifc_param.ifc.vlan.swid = SPECTRUM_SWID
    ifc_param.ifc.vlan.vlan = vlan
    
    ifc_attr = sx_interface_attributes_t()
    ifc_attr.mtu = mtu
    ifc_attr.qos_mode = SX_ROUTER_QOS_MODE_NOP
    ifc_attr.multicast_ttl_threshold = 255
    ifc_attr.loopback_enable = True
    
    rif_p = new_sx_router_interface_t_p()
    rc = sx_api_router_interface_set(handle, SX_ACCESS_CMD_ADD, vrid, ifc_param, ifc_attr, rif_p)
    assert SX_STATUS_SUCCESS == rc, "Failed to create loopback rif"
    
    rif =  sx_router_interface_t_p_value(rif_p)
    print "Created loopback rif: %d, rc: %d " % (rif, rc)
    
    return rif

###################################################### 
#    Structs Function
######################################################
'''
This part contains all the calls to functions that create the necessary structs
'''
""" ############################################################################################ """

def make_sx_ip_addr_v4(addr):
    " This function creates ipv4 sx_ip_addr struct with given ip address. "

    ip_addr = sx_ip_addr_t()
    ip_addr.version = SX_IP_VERSION_IPV4
    ip_addr.addr.ipv4.s_addr = struct.unpack('>I', socket.inet_pton(socket.AF_INET, addr))[0]

    return ip_addr

""" ############################################################################################ """

def make_tunnel_general_params():
    
    general_params = sx_tunnel_general_params_t()
    
    general_params.nve.encap_sport = 0 
    general_params.nve.encap_flowlabel = 0

    general_params.ipinip.encap_flowlabel = 0
    general_params.ipinip.encap_gre_hash = 0

    return general_params
    
""" ############################################################################################ """

def make_tunnel_attributes_vxlan(vrid , underlay_sip ,direction = SX_TUNNEL_DIRECTION_ENCAP):
    
    tunnel_attribute = sx_tunnel_attribute_t()
    
    tunnel_attribute.type = SX_TUNNEL_TYPE_NVE_VXLAN 
    tunnel_attribute.direction = direction 

    #NVE - vxlan attributes
    tunnel_attribute.attributes.vxlan.encap.underlay_vrid = vrid
    tunnel_attribute.attributes.vxlan.encap.underlay_ttl = 255 
    tunnel_attribute.attributes.vxlan.encap.underlay_sip = underlay_sip

    tunnel_attribute.attributes.vxlan.decap.reserved = 0
    tunnel_attribute.attributes.vxlan.nve_log_port = PORT_NVE

    return tunnel_attribute 

###################################################### 
#    Example Function
######################################################
'''
This part contains all the calls to functions that demonstrate the example flows
'''
""" ############################################################################################ """

def bridge_vport_create_example(vlan, port1):
    #create a vport for port 0x10100 and vlan 20
    log_vport_p_1 = new_sx_port_log_id_t_p()
    vport_add_delete(SX_ACCESS_CMD_ADD, port1, vlan, log_vport_p_1)
    log_vport_1 = sx_port_log_id_t_p_value(log_vport_p_1)
    print("virtula port 0x%x created" %(log_vport_1))

    #create bridge
    bridge_id_p = new_sx_bridge_id_t_p()
    bridge_create_delete(SX_ACCESS_CMD_CREATE, bridge_id_p)
    bridge_id = sx_bridge_id_t_p_value(bridge_id_p)
    print("bridge %d created" %(bridge_id))

    #add log_vport_1 to bridge
    bridge_vport_add_delete(SX_ACCESS_CMD_ADD, bridge_id, log_vport_1)
    print ("virtual port 0x%x added to bridge %d " % ( log_vport_1, bridge_id) )

    #set port state to UP
    port_state_set(log_vport_1, SX_PORT_ADMIN_STATUS_UP)

    add_ports_to_vlan(vlan, {PORT1: SX_UNTAGGED_MEMBER})    

    return bridge_id,log_vport_1

def remove_nve_bridge(vlan, port1, bridge_id, log_vport):
    # set port state to Down
    remove_ports_from_vlan(vlan, {PORT1: SX_UNTAGGED_MEMBER})
    port_state_set(log_vport, SX_PORT_ADMIN_STATUS_DOWN)

    # remove log_vport_1 from bridge
    bridge_vport_add_delete(SX_ACCESS_CMD_DELETE, bridge_id, log_vport)
    print ("virtual port 0x%x added to bridge %d " % ( log_vport, bridge_id) )

    bridge_id_p = new_sx_bridge_id_t_p()
    sx_bridge_id_t_p_assign(bridge_id_p, bridge_id)
    bridge_create_delete(SX_ACCESS_CMD_DESTROY, bridge_id_p)
    print("bridge %d removed" %(bridge_id))

    log_vport_p_1 = new_sx_port_log_id_t_p()
    sx_port_log_id_t_p_assign(log_vport_p_1, log_vport)
    vport_add_delete(SX_ACCESS_CMD_DELETE, port1, vlan, log_vport_p_1)
    print("virtula port 0x%x removed" % (log_vport))

    return bridge_id

def example_tunnel_init_flow():
    
    general_param = make_tunnel_general_params()
    general_param_p = new_sx_tunnel_general_params_t_p()
    sx_tunnel_general_params_t_p_assign(general_param_p, general_param)
    tunnel_init(general_param_p)
 
""" ############################################################################################ """

def example_vxlan_tunnel_create_flow(vrid, direction = SX_TUNNEL_DIRECTION_ENCAP):
    underlay_sip = sx_ip_addr_t()
    underlay_sip = make_sx_ip_addr_v4("192.168.1.2") # need to change to valid ip and to correctly add to the struct
    
    tunnel_attribute = make_tunnel_attributes_vxlan(vrid ,underlay_sip, direction)
    tunnel_attribute_p = new_sx_tunnel_attribute_t_p()
    sx_tunnel_attribute_t_p_assign(tunnel_attribute_p ,tunnel_attribute)

    tunnel_id = tunnel_create(tunnel_attribute_p)
    bridge_id,log_vport_1 = prepare_nve_bridge(tunnel_id, VLAN_NVE)

    return tunnel_id,bridge_id,log_vport_1

""" ############################################################################################ """

def example_tunnel_destroy_flow(tunnel_id, bridge_id, log_vport_1, delete_tunnel_after_entry = True):
    tunnel_id_p = new_sx_tunnel_id_t_p()
    sx_tunnel_id_t_p_assign(tunnel_id_p, tunnel_id)

    if delete_tunnel_after_entry == True:
        tunnel_map_entry_delete(tunnel_id, bridge_id)

    rc = tunnel_destroy(tunnel_id_p, delete_tunnel_after_entry)

    if delete_tunnel_after_entry == True:
        remove_nve_bridge(VLAN_NVE, PORT1, bridge_id, log_vport_1)


""" ############################################################################################ """

def init_parser():
    '''
    @summary: init parser
    '''
    usage = "usage: %prog [options] arg, script will run python test with traffic.\n"
    usage += "example: %prog --host_ip=10.212.152.5 --switch_ip=10.7.144.245"
    
    parser = OptionParser(usage=usage)

    parser.add_option('--tunnel' , dest='tunnel_type',default="vxlan" ,
        help="Insert the tunnel type. example --tunnel= vxlan  ------ default = vxlan")
    
    parser.add_option('--traffic', dest='traffic',
        action="store_true" ,default=False ,help="if the test run with traffic add this flag")    

    ##use logger see examlpe in 

    (options, args) = parser.parse_args()
     
    #if not options.tunnel_type:
    #   parser.error('tunnel_type not given, for more info run script with --help')
    #    sys.exit(1)
        
    return options

def test_fdb_static_entry_cnfgr_flow (good_flow = True, tunnel_exsits = True,
                                      tunnel_support_cap = True, proper_dst_type = True,
                                      unique_entry = True, valid_fid = True,
                                      delete_once = True, delete_tunnel_after_entry = True,
                                      entry_delete_args_are_valid = True):

    #vrid = SPECTRUM_SWID
    vrid = create_vrid()

    #init tunnel
    example_tunnel_init_flow()

    #vxlan tunnel create example
    
    if tunnel_support_cap == True:
        tunnel_id, bridge_id, log_vport_1 = example_vxlan_tunnel_create_flow(vrid)
    else:
        # error flow [2]
        tunnel_id, bridge_id, log_vport_1 = example_vxlan_tunnel_create_flow(vrid, SX_TUNNEL_DIRECTION_DECAP)

    #create fdb entry

    mac_list_p = new_sx_fdb_uc_mac_addr_params_t_arr(2)
    data_cnt_p = new_uint32_t_p()
    uint32_t_p_assign(data_cnt_p, 1)
    mac_addr = ether_addr("00:02:c9:11:ad:c1")

    mac_entry = sx_fdb_uc_mac_addr_params_t()
    mac_entry.mac_addr = mac_addr

    # error flow [6]
    if valid_fid == True:
        mac_entry.fid_vid = bridge_id          # Filtering Identifier, VID for static MAC
    else:
        mac_entry.fid_vid = 999

    mac_entry.log_port = 0
    mac_entry.entry_type = SX_FDB_UC_STATIC
    mac_entry.action = SX_FDB_ACTION_FORWARD

    # error flow [3]
    if proper_dst_type == True:
        mac_entry.dest_type = SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP
    else:
        mac_entry.dest_type = SX_FDB_UC_MAC_ADDR_DEST_TYPE_LOGICAL_PORT

    mac_entry.dest.next_hop.next_hop_key.type = SX_NEXT_HOP_TYPE_TUNNEL_ENCAP
    if tunnel_exsits == True:
        mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id = tunnel_id
    else:
        mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id = 1 # any tunnel id

    ip_addr = sx_ip_addr_t()
    ip_addr = make_sx_ip_addr_v4("16.16.16.2")
    mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.underlay_dip = ip_addr
    mac_entry.dest.next_hop.next_hop_data.action = SX_ROUTER_ACTION_FORWARD

    sx_fdb_uc_mac_addr_params_t_arr_setitem(mac_list_p, 0, mac_entry)

    rc = sx_api_fdb_uc_mac_addr_set(handle2, SX_ACCESS_CMD_ADD, 0, mac_list_p, data_cnt_p)
    if good_flow == True:
        # good flow
        assert SX_STATUS_SUCCESS == rc, "UC FDB Failed to add entry"
    elif tunnel_exsits == False:
        # error flow [1]
        assert SX_STATUS_PARAM_ERROR == rc, "UC FDB Failed should return failure because tunnel is not exist"
    elif tunnel_support_cap == False:
        # error flow [2]
        assert SX_STATUS_UNSUPPORTED == rc, "UC FDB Failed should return failure because tunnel cap don't support FDB"
    elif proper_dst_type == False:
        # error flow [3]
        assert SX_STATUS_PARAM_ERROR == rc, "UC FDB Failed should return failure because wrong dst_type is specified"
    elif ((good_flow == False) and (unique_entry == False)):
        # error flow [4]
        assert SX_STATUS_SUCCESS == rc, "UC FDB Failed should return failure because this is duplicated entry"
    elif valid_fid == False:
        # error flow [5]
        assert SX_STATUS_NO_RESOURCES == rc, "UC FDB Failed should return failure because FID is invalid"
    elif delete_tunnel_after_entry == False:
        # error flow [8]
        assert SX_STATUS_SUCCESS == rc, "UC FDB Failed should return success"
    else:
        assert SX_STATUS_SUCCESS == rc, "UC FDB Failed to add entry"

    data_cnt = uint32_t_p_value(data_cnt_p)
    print("sx_api_fdb_uc_mac_addr_set added %d enties, rc: %d " %(data_cnt, rc))

    #print fdb entry
    swid = 0
    mac_entry_arr = new_sx_fdb_uc_mac_addr_params_t_arr(64)
    key_p = new_sx_fdb_uc_mac_addr_params_t_p()
    key_filter_p = new_sx_fdb_uc_key_filter_t_p()
    data_cnt_p = new_uint32_t_p()
    uint32_t_p_assign(data_cnt_p,64)

    rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GET_FIRST, SX_FDB_UC_STATIC, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
    data_cnt = uint32_t_p_value(data_cnt_p)
    print ("sx_api_fdb_uc_mac_addr_get data_cnt:%d , rc %d " % (data_cnt, rc) )
    if (rc != 0):
        print "An error was found in sx_api_fdb_uc_mac_addr_get.\nThe example will exit"
        exit()

    read_number = 0
    while (data_cnt == 64):
        for i in range(0, data_cnt):
            mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
            print ("mac entry %d:" % (read_number*64 + i))
            if ((mac_entry.dest_type == SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP)
                and (mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id == tunnel_id)):
                print_all_attr_from_mac_entry(mac_entry)

        key_p.fid_vid = mac_entry.fid_vid
        key_p.mac_addr = mac_entry.mac_addr
        rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GETNEXT, SX_FDB_UC_ALL, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
        print ("sx_api_fdb_uc_mac_addr_get: rc %d " % ( rc) )
        data_cnt = uint32_t_p_value(data_cnt_p)
        read_number = read_number + 1

    for i in range(0, data_cnt):
        print ("mac entry %d:" % (read_number*64 + i))
        mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
        print_all_attr_from_mac_entry(mac_entry)

    # error flow [7]
    if delete_tunnel_after_entry == False:
        example_tunnel_destroy_flow(tunnel_id, bridge_id, log_vport_1, delete_tunnel_after_entry)

    # remove fdb entry
    if (((unique_entry == True) or ((good_flow == False) and (unique_entry == False)))):
        
        if entry_delete_args_are_valid == True:
            uint32_t_p_assign(data_cnt_p, 1)
            rc = sx_api_fdb_uc_mac_addr_set(handle2, SX_ACCESS_CMD_DELETE, 0, mac_list_p, data_cnt_p)
            data_cnt = uint32_t_p_value(data_cnt_p)
            print("sx_api_fdb_uc_mac_addr_set removed %d enties, rc: %d " %(data_cnt, rc))
        else:
            # error flow [8]
            mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id = 0
            sx_fdb_uc_mac_addr_params_t_arr_setitem(mac_list_p, 0, mac_entry)
            uint32_t_p_assign(data_cnt_p, 1)
            rc = sx_api_fdb_uc_mac_addr_set(handle2, SX_ACCESS_CMD_DELETE, 0, mac_list_p, data_cnt_p)
            assert SX_STATUS_SUCCESS == rc, "UC FDB should not return failure on delete with invalid tunnel"

            mac_entry.dest.next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id = tunnel_id
            sx_fdb_uc_mac_addr_params_t_arr_setitem(mac_list_p, 0, mac_entry)
            uint32_t_p_assign(data_cnt_p, 1)
            rc = sx_api_fdb_uc_mac_addr_set(handle2, SX_ACCESS_CMD_DELETE, 0, mac_list_p, data_cnt_p)
            data_cnt = uint32_t_p_value(data_cnt_p)
            print("sx_api_fdb_uc_mac_addr_set removed %d enties, rc: %d " %(data_cnt, rc))          

        # error flow [6]
        if delete_once == False:
            rc = sx_api_fdb_uc_mac_addr_set(handle2, SX_ACCESS_CMD_DELETE, 0, mac_list_p, data_cnt_p)
            assert SX_STATUS_SUCCESS == rc, "UC FDB Failed should return failure because entry doesn't exist"

    #tunnel destroy
    example_tunnel_destroy_flow(tunnel_id, bridge_id, log_vport_1)

    #deinit tunnel
    tunnel_deinit()

    delete_vrid(vrid)

#######################################################
################# Error flows section #################
#######################################################

######################################################
# 1. Create static FDB entry with non-existing tunnel
#    1.1. Tunnel exists
#    1.2. Tunnel non-exists
#    1.3  Tunnel exists
######################################################
def error_flow_fdb_tunnel_not_exists ():
    print "1. Create static FDB entry with non-existing tunnel"
    print "1.1. Tunnel exists"
    test_fdb_static_entry_cnfgr_flow();
    print "1.2. Tunnel non-exists"
    test_fdb_static_entry_cnfgr_flow(False, False);
    print "1.3  Tunnel exists"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 2. Create static FDB entry with incompatible tunnel
#    2.1. Tunnel capability supports FDB
#    2.2. Tunnel capability isn't support FDB
#    2.3. Tunnel capability supports FDB
######################################################
def error_flow_fdb_tunnel_capability ():
    print "2. Create static FDB entry with incompatible tunnel"
    print "2.1. Tunnel capability supports FDB"
    test_fdb_static_entry_cnfgr_flow();
    print "2.2. Tunnel capability isn't support FDB"
    test_fdb_static_entry_cnfgr_flow(False, True, False);
    print "2.3. Tunnel capability supports FDB"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 3. Create static FDB entry with incompatible destenation type
#    3.1. Destenation type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP
#    3.2. Destenation type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_LOGICAL_PORT
#    3.3. Destenation type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP
######################################################
def error_flow_fdb_tunnel_dest_type ():
    print "4. Create static FDB entry with incompatible destination type"
    print "4.1. Destination type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP"
    test_fdb_static_entry_cnfgr_flow();
    print "4.2. Destination type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_LOGICAL_PORT"
    test_fdb_static_entry_cnfgr_flow(False, True, True, False);
    print "4.3. Destination type is SX_FDB_UC_MAC_ADDR_DEST_TYPE_NEXT_HOP"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 4. Create duplicated static FDB entry
#    4.1. Unique static FDB entry
#    4.2. Duplicated static FDB entry
#    4.3. Unique static FDB entry
######################################################
def error_flow_fdb_entry_duplication ():
    print "4. Create duplicated static FDB entry"
    print "4.1. Unique static FDB entry"
    test_fdb_static_entry_cnfgr_flow();
    print "4.2. Duplicated static FDB entry"
    test_fdb_static_entry_cnfgr_flow(True, True, True, True, False);
    test_fdb_static_entry_cnfgr_flow(False, True, True, True, False);
    print "4.3. Unique static FDB entry"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 5. Create static FDB entry with invalid FID
#    5.1. Valid FID
#    5.2. Valid invalid FID
#    5.3. Valid FID
######################################################
def error_flow_fdb_invalid_fid ():
    print "5. Create static FDB entry with invalid FID"
    print "5.1. Valid FID"
    test_fdb_static_entry_cnfgr_flow();
    print "5.2. Invalid FID"
    test_fdb_static_entry_cnfgr_flow(False, True, True,
                                     True, True, False);
    print "5.3. Valid FID"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 6. Delete static FDB entry two times
#    6.1. Delete static FDB entry one time
#    6.2. Delete static FDB entry two times
#    6.3. Delete static FDB entry one time
######################################################
def error_flow_fdb_two_times_remove ():
    print "6. Delete static FDB entry two times"
    print "6.1. Delete static FDB entry one time"
    test_fdb_static_entry_cnfgr_flow();
    print "6.2. Delete static FDB entry two times"
    test_fdb_static_entry_cnfgr_flow(True, True, True, True,
                                     True, True, False);
    print "6.3. Delete static FDB entry one time"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 7. Delete tunnel when static FDB entry is still attached
#    7.1. Delete tunnel after FDB entry removal
#    7.2. Delete tunnel before FDB entry removal
#    7.3. Delete tunnel after FDB entry removal
######################################################
def error_flow_fdb_remove_tunnel_when_entry_exists ():
    print "7. Delete tunnel when static FDB entry is still attached"
    print "7.1. Delete tunnel after FDB entry removal"
    test_fdb_static_entry_cnfgr_flow();
    print "7.2. Delete tunnel before FDB entry removal"
    test_fdb_static_entry_cnfgr_flow(True, True, True, True,
                                     True, True, True, False);
    print "7.3. Delete tunnel after FDB entry removal"
    test_fdb_static_entry_cnfgr_flow();

######################################################
# 8. Delete FDB entry with invalid arguments
#    8.1. Valid tunnel id
#    8.2. Invalid tunnel id
#    8.3. Valid tunnel id
######################################################
def error_flow_fdb_remove_entry_with_wrong_arg ():
    print "8. Delete FDB entry with invalid arguments"
    print "8.1. Valid tunnel id"
    test_fdb_static_entry_cnfgr_flow();
    print "8.2. Invalid tunnel id"
    test_fdb_static_entry_cnfgr_flow(True, True, True, True, True,
                                     True, True, True, False);
    print "8.3. Valid tunnel id"
    test_fdb_static_entry_cnfgr_flow();

def error_flows_main ():
    error_flow_fdb_tunnel_not_exists()
    error_flow_fdb_tunnel_capability()
    error_flow_fdb_tunnel_dest_type()
    #error_flow_fdb_entry_duplication()
    error_flow_fdb_invalid_fid()
    error_flow_fdb_two_times_remove()
    error_flow_fdb_remove_tunnel_when_entry_exists()
    error_flow_fdb_remove_entry_with_wrong_arg()

######################################################
#    main
######################################################
def main():
    
    #sx_api_tunnel_log_verbosity_level_set(handle,2,5,5)

    print "[LANDMARK] STARTED - ROUTER INIT"
    router_init()
    print "[LANDMARK] COMPLETED - ROUTER INIT"

    options = init_parser()

    print "[LANDMARK] STARTED - ERROR FLOWS TESTING"
    error_flows_main()
    print "[LANDMARK] COMPLETED - ERROR FLOWS TESTING"

    print "\r\n\r\n    Success! All the tests were processed.\r\n\r\n"

if __name__ == "__main__":
    main()



""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
