#!/usr/bin/env python
'''
This file contains Python command example for the FDB DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''
import sys, errno
import os
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *
from pprint import pprint

entry_dict = {0 : 'STATIC', 1 : 'REMOTE', 2 : 'NONAGEABLE', 3 : 'AGEABLE', 4 : 'ALL', \
               5 : 'AGED', 6 : 'NONAGED', 7 : 'POLICY_INVALID'}
action_dict = {0 : 'FORWARD', 1 : 'MIRROR_TO_CPU', 2 : 'TRAP', 3 : 'FORWARD_TO_ROUTER', \
               4 : 'INVALID', 5 : 'INVALID', 6 : 'INVALID', 7 : 'INVALID', 8 : 'INVALID', 9 : 'INVALID', 10 : 'INVALID', 11 : 'INVALID', \
               12 : 'INVALID', 13 : 'INVALID', 14 : 'INVALID', 15 : 'DISCARD'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'


def print_all_attr_from_mac_entry(mac_entry):
    for attr in dir(mac_entry):
        if not attr.startswith("__") and not attr == "this":
            if attr == "mac_addr":
                print "\tmac_addr = %s" % (mac_entry.mac_addr.to_str())
            elif attr == "log_port":
                print "\tlog_port = 0x%x" % (getattr(mac_entry, attr))
            elif attr == "entry_type":
                print "\tentry_type = %s" % entry_dict[(getattr(mac_entry, attr))]
            elif attr == "action":
                print "\taction = %s" % action_dict[(getattr(mac_entry, attr))]
            else:
                print "\t%s = %s" % (attr, getattr(mac_entry, attr))

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
    
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)



swid = 0
mac_entry_arr = new_sx_fdb_uc_mac_addr_params_t_arr(64)
key_p = new_sx_fdb_uc_mac_addr_params_t_p()
key_filter_p = new_sx_fdb_uc_key_filter_t_p()
data_cnt_p = new_uint32_t_p()
uint32_t_p_assign(data_cnt_p,64)

rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GET_FIRST, SX_FDB_UC_ALL, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
data_cnt = uint32_t_p_value(data_cnt_p)
print ("sx_api_fdb_uc_mac_addr_get port_cnt:%d , rc %d " % (data_cnt, rc) )
if (rc != 0):
    print "An error was found in sx_api_fdb_uc_mac_addr_get.\nThe example will exit"
    exit()

read_number = 0
while (data_cnt == 64):
    for i in range(0, data_cnt):
        mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
        print ("mac entry %d:" % (read_number*64 + i))
        print_all_attr_from_mac_entry(mac_entry)
 
    key_p.fid_vid = mac_entry.fid_vid
    key_p.mac_addr = mac_entry.mac_addr
    rc = sx_api_fdb_uc_mac_addr_get(handle, swid, SX_ACCESS_CMD_GETNEXT, SX_FDB_UC_ALL, key_p, key_filter_p, mac_entry_arr, data_cnt_p)
    print ("sx_api_fdb_uc_mac_addr_get: rc %d " % ( rc) )
    data_cnt = uint32_t_p_value(data_cnt_p)
    read_number = read_number + 1
    
for i in range(0, data_cnt):
    print ("mac entry %d:" % (read_number*64 + i))
    mac_entry = sx_fdb_uc_mac_addr_params_t_arr_getitem(mac_entry_arr, i)
    print_all_attr_from_mac_entry(mac_entry)









