#!/usr/bin/env python
"""
This file contains a python commands example for the LAG module.
Python commands syntax is very similar to the SwitchX SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below performs the following configuration operations:
1. Configures the fields which impact the LAG hash distribution function.
2. Creates a new LAG.
3. Adds 2 ports to the above LAG.
4. Enables collection on a specific port in a LAG port.
5. Retrieves the collection mode of a specific port in a LAG port.
6. Enables and disables distribution on a specific port in a LAG port.
7. Retrieves the distribution mode of a specific port in a LAG port.
8. Retrieves the above mentioned LAG information.
9. Deletes 2 ports from the above mentioned LAG.
10. Destroys the LAG.
"""

import sys, errno
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

# 1. Configures the fields which impact the LAG hash distribution function.
lag_hash_param = sx_lag_hash_param_t()
lag_hash_param.lag_hash_type = SX_LAG_HASH_TYPE_XOR
lag_hash_param.lag_hash = 0xA
lag_hash_param.lag_seed = 0
rc = sx_api_lag_hash_flow_params_set(handle, lag_hash_param)
print ("sx_api_lag_hash_flow_params_set , type %d , rc %d " % (lag_hash_param.lag_hash_type, rc) )

# 2. Creates a new LAG.
swid = 0
lag_id_p = new_sx_port_log_id_t_p()
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_CREATE, swid, lag_id_p, None, 0)
lag_id = sx_port_log_id_t_p_value(lag_id_p)
print ("sx_api_lag_port_group_set CREATE lag_id 0x%x , rc %d " % (lag_id, rc) )

rc = sx_api_vlan_port_ingr_filter_set(handle, lag_id, SX_INGR_FILTER_ENABLE)
print ("sx_api_vlan_port_ingr_filter_set SX_INGR_FILTER_ENABLE lag_id 0x%x , rc %d " % (lag_id, rc) )

# 3. Adds 2 ports to the above LAG.
swid = 0
port_list = new_sx_port_log_id_t_arr(2)
sx_port_log_id_t_arr_setitem(port_list, 0, 0x10100)
sx_port_log_id_t_arr_setitem(port_list, 1, 0x10300)
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_ADD, swid, lag_id_p, port_list, 2)
print ("sx_api_lag_port_group_set ADD ports 0x%x, 0x%x to lag_id 0x%x , rc %d " % (0x10100, 0x10300, lag_id, rc) )

# 4. Enables collection on a specific port in a LAG port.
log_port = 0x10300
col_state = COLLECTOR_ENABLE
rc = sx_api_lag_port_collector_set(handle, lag_id, log_port, col_state)
print ("sx_api_lag_port_collector_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, col_state, rc) )

# 5. Retrieves the collection mode of a specific port in a LAG port.
log_port = 0x10300
col_state_p = new_sx_collector_mode_t_p()
rc = sx_api_lag_port_collector_get(handle, lag_id, log_port, col_state_p)
col_state = sx_collector_mode_t_p_value(col_state_p)
print ("sx_api_lag_port_collector_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, col_state, rc) )


# 6. Enables and disables distribution on a specific port in a LAG port.
log_port = 0x10300
distr_state = DISTRIBUTOR_ENABLE
rc = sx_api_lag_port_distributor_set(handle, lag_id, log_port, distr_state)
print ("sx_api_lag_port_distributor_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, distr_state, rc) )

# 6. Enables and disables distribution on a specific port in a LAG port.
distr_state = DISTRIBUTOR_DISABLE
rc = sx_api_lag_port_distributor_set(handle, lag_id, log_port, distr_state)
print ("sx_api_lag_port_distributor_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, distr_state, rc) )

# 7. Retrieves the distribution mode of a specific port in a LAG port.
log_port = 0x10300
distr_state_p = new_sx_distributor_mode_t_p()
rc = sx_api_lag_port_distributor_get(handle, lag_id, log_port, distr_state_p)
distr_state = sx_distributor_mode_t_p_value(distr_state_p)
print ("sx_api_lag_port_distributor_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id, log_port, distr_state, rc) )

# 8. Retrieves the above mentioned LAG information.
swid = 0
#port_list = new_sx_port_log_id_t_arr(2)
sx_port_log_id_t_arr_setitem(port_list, 0, 0)
sx_port_log_id_t_arr_setitem(port_list, 1, 0)
port_list_cnt_p = new_uint32_t_p()
uint32_t_p_assign(port_list_cnt_p, 2)
rc = sx_api_lag_port_group_get(handle, swid, lag_id, port_list, port_list_cnt_p)
port_list_cnt = uint32_t_p_value(port_list_cnt_p)
log_port0 = sx_port_log_id_t_arr_getitem(port_list, 0)
log_port1 = sx_port_log_id_t_arr_getitem(port_list, 1)
print ("sx_api_lag_port_group_get %d ports 0x%x, 0x%x belong to lag_id 0x%x , rc %d " % (port_list_cnt,log_port0 ,log_port1, lag_id, rc) )

# Set LAG HASH type XOR 
lag_hash_param = sx_lag_hash_param_t()
lag_hash_param.lag_hash_type = SX_LAG_HASH_TYPE_XOR
lag_hash_param.lag_hash = 0xA
lag_hash_param.lag_seed = 0
rc = sx_api_lag_hash_flow_params_set(handle, lag_hash_param)
print ("sx_api_lag_hash_flow_params_set , type %d , rc %d " % (lag_hash_param.lag_hash_type, rc) )

# 9. Set distribution list to 2 ports 
fg_params = sx_lag_fine_grain_params_t()
fg_params.resolution = 512
fg_params_p = new_sx_lag_fine_grain_params_t_p()
sx_lag_fine_grain_params_t_p_assign(fg_params_p, fg_params)
fg_port_cnt = 2
fg_port_list = new_sx_lag_fine_grain_member_t_arr(fg_port_cnt)
fg_port = sx_lag_fine_grain_member_t()
fg_port.log_port = 0x10100
fg_port.weight = 10;
sx_lag_fine_grain_member_t_arr_setitem(fg_port_list, 0, fg_port)
fg_port.log_port = 0x10300
fg_port.weight = 20;
sx_lag_fine_grain_member_t_arr_setitem(fg_port_list, 1, fg_port)

rc = sx_api_lag_distributer_list_set(handle, SX_ACCESS_CMD_SET, lag_id, fg_params_p, fg_port_list, fg_port_cnt)
print ("sx_api_lag_distributer_list_set (SET), lag_id 0x%x, rc %d " % (lag_id, rc) )

# 10, Deletes 2 ports from the above mentioned LAG.
swid = 0
sx_port_log_id_t_arr_setitem(port_list, 0, 0x10100)
sx_port_log_id_t_arr_setitem(port_list, 1, 0x10300)
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DELETE, swid, lag_id_p, port_list, 2)
print ("sx_api_lag_port_group_set DELETE ports 0x%x, 0x%x from lag_id 0x%x , rc %d " % (0x10100, 0x10300, lag_id, rc) )

rc = sx_api_lag_distributer_list_set(handle, SX_ACCESS_CMD_DELETE_ALL, lag_id, None, None, 0)
print ("sx_api_lag_distributer_list_set (DELETE_ALL), lag_id 0x%x, rc %d " % (lag_id, rc) )

# 11. Destroys the LAG.
swid = 0
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DESTROY, swid, lag_id_p, None, 0)
lag_id = sx_port_log_id_t_p_value(lag_id_p)
print ("sx_api_lag_port_group_set DESTROY lag_id 0x%x , rc %d " % (lag_id, rc) )

# 12. cleanup allocated objects 
delete_sx_port_log_id_t_p(lag_id_p)
delete_sx_port_log_id_t_arr(port_list)
delete_sx_collector_mode_t_p(col_state_p) 
delete_sx_lag_fine_grain_params_t_p(fg_params_p)
delete_sx_lag_fine_grain_member_t_arr(fg_port_list)


