#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys, errno
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

log_port = 0x10100
rc = sx_api_port_init_set(handle, log_port)
print ("sx_api_port_init_set log_port 0x%x , rc %d " % ( log_port, rc) )

swid_list_p = new_sx_swid_t_arr(16)
swid_cnt_p = new_uint32_t_p()
uint32_t_p_assign(swid_cnt_p, 16)
rc = sx_api_port_swid_list_get(handle, swid_list_p, swid_cnt_p)
swid_cnt = uint32_t_p_value(swid_cnt_p)
print ("sx_api_port_swid_list_get swid_cnt:%d , rc %d, swid table: " % ( swid_cnt, rc) )
for i in range(0,swid_cnt):
    print ("[%d] swid %d" % ( i, sx_swid_t_arr_getitem(swid_list_p, i)) )

swid = 0
log_port_list_p = new_sx_port_log_id_t_arr(64)
port_cnt_p = new_uint32_t_p()
uint32_t_p_assign(port_cnt_p,64)
rc = sx_api_port_swid_port_list_get(handle, swid, log_port_list_p, port_cnt_p);
port_cnt = uint32_t_p_value(port_cnt_p)
print ("sx_api_port_swid_port_list_get port_cnt:%d , rc %d, port table: " % ( port_cnt, rc) )
for i in range(0,port_cnt):
    print ("[%d] port 0x%x" % ( i, sx_port_log_id_t_arr_getitem(log_port_list_p, i)) )

log_port = 0x10100
swid_p = new_sx_swid_t_p()
rc = sx_api_port_swid_bind_get(handle, log_port,  swid_p);
swid = sx_swid_t_p_value(swid_p)
print ("sx_api_port_swid_bind_get log_port:0x%x , swid: %d, rc %d " % ( log_port, swid, rc) )

base_mac_addr_p = new_sx_mac_addr_t_p()
mac_addr1 = ether_addr(0,2,3,4,5,6)
sx_mac_addr_t_p_assign(base_mac_addr_p, mac_addr1)
port_attributes_list = new_sx_port_attributes_t_arr(64)
port_attributes = sx_port_attributes_t()
port_attributes.port_mode = SX_PORT_MODE_EXTERNAL
port_attributes.port_mapping.local_port = 1
port_attributes.port_mapping.mapping_mode = SX_PORT_MAPPING_MODE_ENABLE
port_attributes.port_mapping.module_port = 8
port_attributes.port_mapping.width = 4
port_attributes.port_mapping.lane_bmap = 15
port_attributes.port_mapping.config_hw = 0
port_attributes.log_port = 0x10100
sx_port_attributes_t_arr_setitem(port_attributes_list,0,port_attributes)
uint32_t_p_assign(port_cnt_p,64)
rc = sx_api_port_device_get(handle, 1 , 0, port_attributes_list,  port_cnt_p)
port_cnt = uint32_t_p_value(port_cnt_p)
print ("sx_api_port_device_get port_cnt:%d , rc %d, port_list : " % (port_cnt, rc) )
print ("=================================================================")
for i in range(0,port_cnt):
    port_attributes = sx_port_attributes_t_arr_getitem(port_attributes_list,i)
    print ("[%d]: log_port:0x%x local_port:%d" % (i,port_attributes.log_port, port_attributes.port_mapping.local_port))
print ("=================================================================")


