#!/usr/bin/env python
'''
This file contains Python command example for the PORT DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''

oper_dict = {0 : 'NONE', 1 : 'UP', 2 : 'DOWN', 3 : 'DOWN_BY_FAIL'} 
admin_dict = {0 : 'NONE', 1 : 'UP', 2 : 'UP_ONCE', 3 : 'DOWN', 4 : 'CONFIGURABLE'} 
module_dict = {0 : 'INITIALIZING', 1 : 'PLUGGED', 2 : 'UNPLUGGED'}        
      
       
import sys , errno
import os
from test_infra_common import *
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')

from sx_api import *

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

port_attributes_list = new_sx_port_attributes_t_arr(64)
port_cnt_p = new_uint32_t_p()
max_mtu_size_p=new_sx_port_mtu_t_p()
oper_mtu_size_p=new_sx_port_mtu_t_p()
uint32_t_p_assign(port_cnt_p,64)
oper_state_p = new_sx_port_oper_state_t_p()
admin_state_p = new_sx_port_admin_state_t_p()
module_state_p = new_sx_port_module_state_t_p()
vid_p = new_sx_vid_t_p()

rc = sx_api_port_device_get(handle, 1 , 0, port_attributes_list,  port_cnt_p)
port_cnt = uint32_t_p_value(port_cnt_p)
print ("sx_api_port_device_get port_cnt:%d , rc %d, port_list : " % (port_cnt, rc) )
if (rc != 0):
    print "An error was found in sx_api_fdb_uc_mac_addr_get.\nThe example will exit"
    exit()
print ("======================================================================================")
header = ["log_port", "local_port", "label_port", "mtu", "admin_s", "oper_s", "module_s", "pvid"]
print "|%10s|%10s|%10s|%10s|%8s|%8s|%15s|%6s|" % (header[0],header[1],header[2],header[3],header[4],header[5],header[6], header[7])
print ("======================================================================================")

for i in range(0,port_cnt):

    port_attributes = sx_port_attributes_t_arr_getitem(port_attributes_list,i)
    is_vport = check_vport(int(port_attributes.log_port))
    is_nve = check_nve(int(port_attributes.log_port))
    if is_nve:
        continue
    else:
        if is_vport:
            sx_port_mtu_t_p_assign(max_mtu_size_p, 0)
            sx_port_mtu_t_p_assign(oper_mtu_size_p, 0)
        else:
            sx_api_port_mtu_get(handle, int(port_attributes.log_port), max_mtu_size_p, oper_mtu_size_p)
        sx_api_port_state_get(handle, int(port_attributes.log_port), oper_state_p, admin_state_p, module_state_p)
        sx_api_vlan_port_pvid_get(handle, int(port_attributes.log_port), vid_p)
    log_port = "0x%x" % port_attributes.log_port
    print ("|%10s|%10d|%10d|%10d|%8s|%8s|%15s|%6s|" % 
                                                                   (log_port, 
                                                                    port_attributes.port_mapping.local_port,
                                                                    port_attributes.port_mapping.module_port + 1,
                                                                    sx_port_mtu_t_p_value(oper_mtu_size_p),
                                                                    admin_dict[sx_port_admin_state_t_p_value(admin_state_p)],
                                                                    oper_dict[sx_port_oper_state_t_p_value(oper_state_p)],
                                                                    module_dict[sx_port_module_state_t_p_value(module_state_p)],
								    								sx_vid_t_p_value(vid_p)
                                                                    ))

print ("======================================================================================")


