#!/usr/bin/env python
'''
This file contains Python command example for the ROUTER ECMP READ module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys, errno
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *
from pprint import pprint

prio_dict = {0 : 'SX_TRAP_PRIORITY_BEST_EFFORT', 1 : 'SX_TRAP_PRIORITY_LOW', 2 : 'SX_TRAP_PRIORITY_MED', 3 : 'SX_TRAP_PRIORITY_HIGH', 4 : 'SX_TRAP_PRIORITY_CRITICAL'}
type_dict = { 1 : 'SX_NEXT_HOP_TYPE_IP'}
action_dict = {0 : 'SX_ROUTER_ACTION_DROP', 1 : 'SX_ROUTER_ACTION_TRAP', 2 : 'SX_ROUTER_ACTION_FORWARD', 3 : 'SX_ROUTER_ACTION_MIRROR', \
               4 : 'SX_ROUTER_ACTION_TRAP_FORWARD', 5 : 'SX_ROUTER_ACTION_SPAN'}
ip_dict = {0 : 'SX_IP_VERSION_NONE', 1 : 'SX_IP_VERSION_IPV4', 2 : 'SX_IP_VERSION_IPV6'}

def print_all_attr_from_a_struct_in_hexa(struct):
    for attr in dir(struct):
        if not attr.startswith("__") and not attr == "this":
            print "\t%s = 0x%x" % (attr, getattr(struct, attr))

def print_all_attr_from_next_hop(next_hop):
    print "\tversion = %s" % ip_dict[next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address.version]
    if ip_dict[next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address.version] == 'SX_IP_VERSION_IPV4':
        print_all_attr_from_a_struct_in_hexa(next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address.addr.ipv4)
    elif ip_dict[next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address.version] == 'SX_IP_VERSION_IPV6':
        print_all_attr_from_a_struct_in_hexa(next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address.addr.ipv6)
    print "\ttype = %s" % type_dict[next_hop.next_hop_key.type]
    print "\taction = %s" % action_dict[next_hop.next_hop_data.action]
    print "\tweight = %s" % next_hop.next_hop_data.weight
    print "\tcounter_id = %s" % next_hop.next_hop_data.counter_id
    print "\ttrap_attr.prio = %s" % prio_dict[next_hop.next_hop_data.trap_attr.prio]
        
#init to the params
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)
    
if len(sys.argv) >= 2:
    ecmp_id = int(str(sys.argv[1]))
else:
    print ("ERROR:\"ecmp id\" arg is missing.\nPlease enter ecmp id as a parameter.")
    exit()
    
next_hop_arr = new_sx_next_hop_t_arr(64)
next_hop_cnt_p = new_uint32_t_p()
uint32_t_p_assign(next_hop_cnt_p,20)
rc = sx_api_router_ecmp_get(handle, ecmp_id, next_hop_arr, next_hop_cnt_p)
if rc != 0:
    print ("ERROR:sx_api_router_ecmp_get got rc:%d\nThis may be because a non existing ecmp id" % (rc) )
    exit()
next_hop_cnt = uint32_t_p_value(next_hop_cnt_p)
print ("sx_api_router_ecmp_get: cnt:%d rc:%d " % ( next_hop_cnt, rc) )
     
for i in range(0, next_hop_cnt):
   print ("next hop %d:" % ( i))
   next_hop = sx_next_hop_t_arr_getitem(next_hop_arr, i)
   print_all_attr_from_next_hop(next_hop)      
