#!/usr/bin/env python
'''
This file contains Python command example for the ALL ROUTER INTERFACE DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
qos_mode_dict = {0 : 'NOP', 1 : 'PRIO_FROM_DSCP', 2 : 'PRESERVE_PRIO'}
egress_mode_dict = {0 : 'TAGGED_MEMBER', 1 : 'UNTAGGED_MEMBER'}
type_dict = {0 : 'VLAN', 1 : 'PORT_VLAN', 2 : 'VPORT', 3 : 'PKEY', 4 : 'BRIDGE_PORT', 5 : 'BRIDGE'}
state_dict = {0 : 'DISABLE', 1 : 'ENABLE', 2 : 'UNKOWN', 3 : 'UNKOWN', 4 : 'UNKOWN'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'
MIN = 0
MAX = 1000

import sys, errno
import os

sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *
from pprint import pprint

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
       
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

vrid_p = new_sx_router_id_t_p()
sx_router_id_t_p_assign(vrid_p, 0)
ifc_param_p = sx_router_interface_param_t()
ifc_attr_p = sx_interface_attributes_t()
ifc_state_p = sx_router_interface_state_t()

tables = []
for rif in range (MIN,MAX):
    rc1 = sx_api_router_interface_get(handle, rif,vrid_p ,ifc_param_p , ifc_attr_p)
    if rc1 == SX_STATUS_MODULE_UNINITIALIZED:
        # Check if router module is initialized.
        print "####################################"
        print "# Router is not initialized " 
        print "####################################"
        exit()
    rc2 = sx_api_router_interface_state_get(handle, rif, ifc_state_p)
    vrid = sx_router_id_t_p_value(vrid_p)
    ifc_param = sx_router_interface_param_t_p_value(ifc_param_p)
    ifc_attr = sx_interface_attributes_t_p_value(ifc_attr_p)
    ifc_state = sx_router_interface_state_t_p_value(ifc_state_p)
    
    if (rc1 == 0 and rc2 == 0):
        type = type_dict[ifc_param.type]
        swid = 0
        vlan = 0
        port = 0
        egress_mode = "0"
        if (type == "VLAN"):
            swid = ifc_param.ifc.vlan.swid
            vlan = ifc_param.ifc.vlan.vlan
        elif (type == "PORT_VLAN"):
            port = ifc_param.ifc.port_vlan.port
            vlan = ifc_param.ifc.port_vlan.vlan   
        elif (type == "VPORT"):
                port = ifc_param.ifc.vport.port
        elif (type == "BRIDGE"):
            port = ifc_param.ifc.bridge.port
        elif (type == "BRIDGE_PORT"):
            port = ifc_param.ifc.bridge_port.port
            vlan = ifc_param.ifc.bridge_port.vlan 
            egress_mode = egress_mode_dict[ifc_param.ifc.bridge_port.egress_mode]
        port = "0x%x" % port
        mac = ifc_attr.mac_addr.to_str()
        mtu = ifc_attr.mtu
        ttl = ifc_attr.multicast_ttl_threshold
        qos_mode = qos_mode_dict[ifc_attr.qos_mode]
        ipv4_en = state_dict[ifc_state.ipv4_enable]
        ipv6_en = state_dict[ifc_state.ipv6_enable]
        ipv4_mc_en = state_dict[ifc_state.ipv4_mc_enable]
        ipv6_mc_en = state_dict[ifc_state.ipv6_mc_enable]
        table = [rif, vrid, type, swid, vlan, port, egress_mode, mac,mtu,ttl,qos_mode,ipv4_en,ipv6_en,ipv4_mc_en,ipv6_mc_en]
        tables.append(table)
header = ["RIF", "Router ID", "TYPE", "L2 SWID", "L2 VLAN", "LOG PORT", "EGRESS_MODE", "L2 MAC", "L2 MTU", "L3 TTL", "QOS", "ipv4 UC state", "ipv6 UC state", "ipv4 MC state", "ipv6 MC state"]
print "----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------"
print "|%7s|%10s|%10s|%10s|%10s|%10s|%12s|%20s|%10s|%10s|%15s|%14s|%14s|%14s|%14s|" % (header[0],header[1],header[2],header[3],header[4],header[5],header[6],header[7],header[8],header[9],header[10],header[11],header[12],header[13],header[14])
print "----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------"
for table in tables:
    print "|%7d|%10d|%10s|%10d|%10d|%10s|%12s|%20s|%10d|%10d|%15s|%14s|%14s|%14s|%14s|" % (table[0],table[1],table[2],table[3],table[4],table[5],table[6],table[7],table[8],table[9],table[10],table[11],table[12],table[13],table[14])
print "----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------"






