#!/usr/bin/env python
'''
This file contains Python command example for the ROUTER NEIGHBORS DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
'''
import sys, errno
import os
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')

from sx_api import *
from pprint import pprint

ip_dict = {0 : 'NONE', 1 : 'IPV4', 2 : 'IPV6'}
prio_dict = {0 : 'BEST_EFFORT', 1 : 'LOW', 2 : 'MED', 3 : 'HIGH', 4 : 'CRITICAL'}
action_dict = {0 : 'DROP', 1 : 'TRAP', 2 : 'FORWARD', 3 : 'TRAP_FORWARD', 4 : 'SPAN'}
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'


def print_all_attr_from_a_struct_in_hexa(struct):
    for attr in dir(struct):
        if not attr.startswith("__") and not attr == "this":
            print "\t%s = 0x%x" % (attr, getattr(struct, attr))
            
def print_all_attr_from_neighbor(neighbor):
    print "\tversion = %s" % ip_dict[neighbor.ip_addr.version]
    if ip_dict[neighbor.ip_addr.version] == 'IPV4':
        print_all_attr_from_a_struct_in_hexa(neighbor.ip_addr.addr.ipv4)
    elif ip_dict[neighbor.ip_addr.version] == 'IPV6':
        print_all_attr_from_a_struct_in_hexa(neighbor.ip_addr.addr.ipv6)
    print "\taction = %s" % action_dict[neighbor.neigh_data.action]
    print "\tmac_addr = %s" % neighbor.neigh_data.mac_addr.to_str()
    print "\trif = %s" % neighbor.neigh_data.rif
    if action_dict[neighbor.neigh_data.action] != "FORWARD":
        print "\ttrap_attr.prio = %s" % prio_dict[neighbor.neigh_data.trap_attr.prio]


print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
    
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

neigh_entry_arr = new_sx_neigh_get_entry_t_arr(64)
rif =  0
neigh_key_p = new_sx_ip_addr_t_p()
neigh_key = sx_ip_addr_t()
neigh_key.version = 0
neigh_key.addr.ipv4.addr = 0
sx_ip_addr_t_p_assign(neigh_key_p, neigh_key)
data_cnt_p = new_uint32_t_p()
uint32_t_p_assign(data_cnt_p,20)
rc = sx_api_router_neigh_get(handle, SX_ACCESS_CMD_GET_FIRST, rif, neigh_key_p, None, neigh_entry_arr, data_cnt_p)
data_cnt = uint32_t_p_value(data_cnt_p)
if rc != 0:
	#check if router module initialize
	if rc == SX_STATUS_MODULE_UNINITIALIZED:
		print "####################################"
		print "# Router is not initialized " 
		print "####################################"
        exit()
	print "An error was found in sx_api_router_neigh_get. rc: %d" % (rc)
	exit()
print ("sx_api_router_neigh_get: cnt:%d rc:%d " % ( data_cnt, rc) )

read_number = 0
while (data_cnt == 20):
    for i in range(0, data_cnt):
        print ("neighbor %d:" % (read_number*20 + i))
        neighbor = sx_neigh_get_entry_t_arr_getitem(neigh_entry_arr, i)
        print_all_attr_from_neighbor(neighbor)
 
    rif = neighbor.neigh_data.rif
    sx_ip_addr_t_p_assign(neigh_key_p, neighbor.ip_addr)        
    rc = sx_api_router_neigh_get(handle, SX_ACCESS_CMD_GETNEXT, rif, neigh_key_p, None, neigh_entry_arr, data_cnt_p)
    print ("sx_api_router_neigh_get: rc %d " % ( rc) )
    data_cnt = uint32_t_p_value(data_cnt_p)
    read_number = read_number + 1
    
for i in range(0, data_cnt):
    print ("neighbor %d:" % (read_number*20 + i))
    neighbor = sx_neigh_get_entry_t_arr_getitem(neigh_entry_arr, i)
    print_all_attr_from_neighbor(neighbor)









