#!/usr/bin/env python
'''
This file contains Python command example for the ALL ROUTE DUMP module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

Usage:
sx_api_router_uc_routes_dump_all.py <num of virtual routers to dump>

default virtual routers num is 12 
'''
import sys, errno
import os
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *
from pprint import pprint

ip_dict = {0 : 'NONE', 1 : 'IPV4', 2 : 'IPV6'}
action_dict = {0 : 'DROP', 1 : 'TRAP', 2 : 'FORWARD', 3 : 'TRAP_FORWARD', 4 : 'SPAN'}
type_dict = {0 : 'NEXT_HOP', 1 : 'LOCAL', 2 : 'IP2ME'}

MIN = 0
DEFAULT_MAX = 12
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'


def print_all_attr_from_a_struct(struct):
	for attr in dir(struct):
		if not attr.startswith("__") and not attr == "this":
			print "\t%s = %s" % (attr, getattr(struct, attr))

def print_all_next_hops(next_hop_list, hop_cnt):
	for i in range(0, hop_cnt):
		print ("\t\thop %d:" % ( i ))
		hop = sx_ip_addr_t_arr_getitem(next_hop_list, i)
		print "\t\t\tversion = %s" % ip_dict[hop.version]
		if ip_dict[hop.version] == 'IPV4':
			print_all_attr_from_a_struct_in_hexa(hop.addr.ipv4, True)
		elif ip_dict[hop.version] == 'IPV6':
			print_all_attr_from_a_struct_in_hexa(hop.addr.ipv6, True)


def print_all_attr_from_a_struct_in_hexa(struct, triple_tab = False):        
	for attr in dir(struct):
		if not attr.startswith("__") and not attr == "this":
			if triple_tab:
				print "\t\t\t%s = 0x%x" % (attr, getattr(struct, attr))
			else:
				print "\t%s = 0x%x" % (attr, getattr(struct, attr))

def print_all_attr_from_route(route):
	print "\tversion = %s" % ip_dict[route.network_addr.version]
	if ip_dict[route.network_addr.version] == 'IPV4':
		print_all_attr_from_a_struct_in_hexa(route.network_addr.prefix.ipv4.addr)
		print_all_attr_from_a_struct_in_hexa(route.network_addr.prefix.ipv4.mask)
	elif ip_dict[route.network_addr.version] == 'IPV6':
		print_all_attr_from_a_struct_in_hexa(route.network_addr.prefix.ipv6.addr)
		print_all_attr_from_a_struct_in_hexa(route.network_addr.prefix.ipv6.mask)
	print "\ttype = %s" % type_dict[route.route_data.type]
	print "\tnext_hop_cnt = %d" % route.route_data.next_hop_cnt
	if (type_dict[route.route_data.type] == "NEXT_HOP" or type_dict[route.route_data.type] == "LOCAL"):
		print "\taction = %s" % action_dict[route.route_data.action]
		if (action_dict[route.route_data.action] == "TRAP" or action_dict[route.route_data.action] == "RAP_FORWARD" ):
			print "\ttrap_attr.prop = %d" % route.route_data.trap_attr.prio
		if (action_dict[route.route_data.action] == "FORWARD" or action_dict[route.route_data.action] == "TRAP_FORWARD" \
			or action_dict[route.route_data.action] == "SPAN"):
			if (type_dict[route.route_data.type] == "NEXT_HOP"):
				print "\tnext hop list:"
				print_all_next_hops(route.route_data.next_hop_list_p, route.route_data.next_hop_cnt)
				print "\tecmp_id = %d" % route.route_data.uc_route_param.ecmp_id
			if (type_dict[route.route_data.type] == "LOCAL"):
				print "\tlocal_egress_rif = %d" % route.route_data.uc_route_param.local_egress_rif

def print_all_routes_of_vrid( vrid ):
	vrid = vrid
	rc = sx_api_router_uc_route_get(handle, SX_ACCESS_CMD_GET_FIRST, vrid, network_addr_p, None, uc_route_arr, data_cnt_p)
	data_cnt = uint32_t_p_value(data_cnt_p)
	if rc != 0:
		#check if router module initialize
		if rc == SX_STATUS_MODULE_UNINITIALIZED:
			print "####################################"
			print "# Router is not initialized " 
			print "####################################"
			exit()
		return

	print ("-----------------------------------------------------------")
	print ("sx_api_router_uc_route_get: cnt:%d rc:%d " % ( data_cnt, rc) )
	print ("vrid %d:" % (vrid))
	read_number = 0
	while (data_cnt == 20):
		for i in range(0, data_cnt):
			print ("route %d:" % (read_number*20 + i))
			route = sx_uc_route_get_entry_t_arr_getitem(uc_route_arr, i) 
			print_all_attr_from_route(route)

		sx_ip_prefix_t_p_assign(network_addr_p, route.network_addr) 
		rc = sx_api_router_uc_route_get(handle, SX_ACCESS_CMD_GETNEXT, vrid, network_addr_p, None, uc_route_arr, data_cnt_p)
		print ("sx_api_router_uc_route_get: rc %d " % ( rc) )
		if rc != 0:
			print "An error was found in sx_api_router_uc_route_get. rc: %d" % (rc)
			exit()
		data_cnt = uint32_t_p_value(data_cnt_p)
		read_number = read_number + 1

	for i in range(0, data_cnt):
		print ("route %d:" % (read_number*20+ i))
		route = sx_uc_route_get_entry_t_arr_getitem(uc_route_arr, i)
		print_all_attr_from_route(route)

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
	os.chmod(ERR_FILE_LOCATION, 0o777);

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)
    
#init to the params
if len(sys.argv) >= 2:
	max = int(str(sys.argv[1]))
else:
	max = DEFAULT_MAX

uc_route_arr = new_sx_uc_route_get_entry_t_arr(64)
network_addr_p = new_sx_ip_prefix_t_p()
network_addr = sx_ip_prefix_t()
network_addr.version = SX_IP_VERSION_IPV4
network_addr.prefix.ipv4.addr.s_addr = 0
sx_ip_prefix_t_p_assign(network_addr_p, network_addr)
data_cnt_p = new_uint32_t_p()

for vrid in range (MIN,max):
        uint32_t_p_assign(data_cnt_p,20)
	print_all_routes_of_vrid(vrid)
