#!/usr/bin/env python
'''
This file contains Python command example for the VRID READ module.
Python commands syntax is very similar to the Switch SDK APIs. 
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
action_dict = {0 : 'DROP', 1 : 'TRAP', 2 : 'FORWARD', 3 : 'MIRROR', \
               4 : 'TRAP_FORWARD', 5 : 'SPAN', 6 : 'DROP', 7 : 'SPAN'}
state_dict = {0 : 'DISABLE', 1 : 'ENABLE', 2 : 'UNKOWN', 3 : 'UNKOWN', 4 : 'UNKOWN'}

MIN = 0
MAX = 256
ERR_FILE_LOCATION = '/tmp/python_err_log.txt'
import sys, errno
import os

sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
from sx_api import *
from pprint import pprint

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);
       
print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

router_attr_p = sx_router_attributes_t()

tables = []
for vrid in range (MIN,MAX):
    rc = sx_api_router_get(handle, vrid, router_attr_p)
    if rc == SX_STATUS_MODULE_UNINITIALIZED:
        #check if router module initialize
        print "####################################"
        print "# Router is not initialized " 
        print "####################################"
        exit()
	
	#print ("sx_api_router_get vrid:%d , rc %d " % ( vrid, rc) )
    if rc == 0:
        router_attr = sx_router_attributes_t_p_value(router_attr_p)
        
        ipv4_enable = state_dict[router_attr.ipv4_enable]
        ipv6_enable = state_dict[router_attr.ipv6_enable]
        ipv4_mc_enable = state_dict[router_attr.ipv4_mc_enable]
        ipv6_mc_enable = state_dict[router_attr.ipv6_mc_enable]
        uc_default_rule_action = action_dict[router_attr.uc_default_rule_action]
        mc_default_rule_action = action_dict[router_attr.mc_default_rule_action]

        table = [vrid, ipv4_enable, ipv6_enable, ipv4_mc_enable, ipv6_mc_enable, uc_default_rule_action, mc_default_rule_action]
        tables.append(table)
header = ["VRID", "IPv4_enable", "IPv6_enable", "IPv4_mc_enable", "IPv6_mc_enable", "UC default rule action", "MC default rule action"]
print "-----------------------------------------------------------------------------------------------------------------------"
print "|%7s|%15s|%15s|%15s|%15s|%22s|%22s|" % (header[0],header[1],header[2],header[3],header[4],header[5],header[6])
print "-----------------------------------------------------------------------------------------------------------------------"
for table in tables:
    print "|%7d|%15s|%15s|%15s|%15s|%22s|%22s|" % (table[0],table[1],table[2],table[3],table[4],table[5],table[6])
print "-----------------------------------------------------------------------------------------------------------------------"






