#!/usr/bin/env python
'''
This file contains a Python commands example for the sflow module.

Python commands syntax is very similar to the Switch SDK APIs.

You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

\section cases Example Flow

The example below performs the following configuration operations:

1. Open host intreface.

2. Set trap group.

3. Set trap id for sflow.

4. Register trap with host interface.

5 Configure LAG hash distribution function

6 Create LAG

7 Set ingress filter

8 Add ports to LAG

9 Enable collection on a specific LAG port

10 Enable distribution on a specific LAG port

11 Create sflow on port

12 Get sflow statistics from port

13 Delete sflow from port

14 Deletes ports from LAG

15 Destroys LAG

16 Cleanup allocated objects 

'''
import sys, errno
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')

import time
import os
import sxd_api
from sx_api import *
from sx_api import *
from pprint import pprint
os.system("command")

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'


def print_all_attr_from_a_struct(struct):
    for attr in dir(struct):
        if not attr.startswith("__") and not attr == "this":
            print "\t%s = %s" % (attr, getattr(struct, attr))
if len(sys.argv) >= 6:
    log_port1 = int(str(sys.argv[1]), 16)
    log_port2 = int(str(sys.argv[2]), 16)
    log_port3 = int(str(sys.argv[3]), 16)
    log_port4 = int(str(sys.argv[4]), 16)
    rate = int(str(sys.argv[5]), 10)
    sleep_cntr = int(str(sys.argv[6]), 10)
elif len(sys.argv) >= 6:
    log_port1 = int(str(sys.argv[1]), 16)
    log_port2 = int(str(sys.argv[2]), 16)
    log_port3 = int(str(sys.argv[3]), 16)
    log_port4 = int(str(sys.argv[4]), 16)
    rate = int(str(sys.argv[5]), 10)
elif len(sys.argv) >= 5:
    log_port1 = int(str(sys.argv[1]), 16)
    log_port2 = int(str(sys.argv[2]), 16)
    log_port3 = int(str(sys.argv[3]), 16)
    log_port4 = int(str(sys.argv[4]), 16)
    rate = 1000
    sleep_cntr = 600
else:
    log_port1 = 0x10100
    log_port2 = 0x10300
    log_port3 = 0x12100
    log_port4 = 0x12300
    rate = 1000
    sleep_cntr = 600

print ("Input parameters: ports 0x%x 0x%x 0x%x 0x%x, rate %d sleep %d "
        % ( log_port1, log_port2, log_port3, log_port4, rate, sleep_cntr) )

print "[+] Redirecting stderr to: %s" % ERR_FILE_LOCATION
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777);

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

# Open host intreface
sx_fd_p = new_sx_fd_t_p()
rc = sx_api_host_ifc_open(handle, sx_fd_p)
sx_fd = sx_fd_t_p_value(sx_fd_p)
print ("sx_api_host_ifc_open 0x%x , rc %d " % ( sx_fd.fd , rc) )

# Set trap group
swid = 0
trap_group = 1
sx_trap_group_attributes_p = new_sx_trap_group_attributes_t_p()
sx_trap_group_attributes = sx_trap_group_attributes_t()
sx_trap_group_attributes.prio = SX_TRAP_PRIORITY_BEST_EFFORT
sx_trap_group_attributes.truncate_mode = SX_TRUNCATE_MODE_DISABLE
sx_trap_group_attributes.truncate_size = 0
sx_trap_group_attributes.control_type = SX_CONTROL_TYPE_DEFAULT
sx_trap_group_attributes_t_p_assign(sx_trap_group_attributes_p, sx_trap_group_attributes)
rc = sx_api_host_ifc_trap_group_set(handle, swid, trap_group, sx_trap_group_attributes_p)
print ("sx_api_host_ifc_trap_group_set prio 0x%x , rc %d " % ( sx_trap_group_attributes.prio , rc) )

# Set trap id for sflow
trap_id = SX_TRAP_ID_ETH_L2_PACKET_SAMPLING
trap_action = SX_TRAP_ACTION_TRAP_2_CPU
rc = sx_api_host_ifc_trap_id_set(handle, swid, trap_id, trap_group, trap_action)
print ("sx_api_host_ifc_trap_id_set rc %d " % (rc) )

# Register trap with host interface
sx_user_channel_p = new_sx_user_channel_t_p()
sx_user_channel = sx_user_channel_t()
sx_user_channel.type = SX_USER_CHANNEL_TYPE_FD
sx_user_channel.channel.fd = sx_fd
sx_user_channel_t_p_assign(sx_user_channel_p, sx_user_channel)
rc = sx_api_host_ifc_trap_id_register_set(handle, SX_ACCESS_CMD_REGISTER, swid, trap_id, sx_user_channel_p)
print ("sx_api_host_ifc_trap_id_register_set  rc %d " % (rc) )

# Configure LAG hash distribution function
lag_hash_param = sx_lag_hash_param_t()
lag_hash_param.lag_hash_type = SX_LAG_HASH_TYPE_XOR
lag_hash_param.lag_hash = 0xA
lag_hash_param.lag_seed = 0
rc = sx_api_lag_hash_flow_params_set(handle, lag_hash_param)
print ("sx_api_lag_hash_flow_params_set , type %d , rc %d " % (lag_hash_param.lag_hash_type, rc) )

# Create LAG
swid = 0
lag_id_p = new_sx_port_log_id_t_p()
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_CREATE, swid, lag_id_p, None, 0)
lag_id = sx_port_log_id_t_p_value(lag_id_p)
print ("sx_api_lag_port_group_set CREATE lag_id 0x%x , rc %d " % (lag_id, rc) )

# Set ingress filter
rc = sx_api_vlan_port_ingr_filter_set(handle, lag_id, SX_INGR_FILTER_ENABLE)
print ("sx_api_vlan_port_ingr_filter_set SX_INGR_FILTER_ENABLE lag_id 0x%x , rc %d " % (lag_id, rc) )

# Add ports to LAG
swid = 0
port_list = new_sx_port_log_id_t_arr(4)
sx_port_log_id_t_arr_setitem(port_list, 0, log_port1)
sx_port_log_id_t_arr_setitem(port_list, 1, log_port2)
sx_port_log_id_t_arr_setitem(port_list, 2, log_port3)
sx_port_log_id_t_arr_setitem(port_list, 3, log_port4)
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_ADD, swid, lag_id_p, port_list, 4)
print ("sx_api_lag_port_group_set ADD ports 0x%x, 0x%x 0x%x, 0x%x  to lag_id 0x%x , rc %d "
       % (log_port1, log_port2, log_port3, log_port4, lag_id, rc) )

# Enables collection on a specific LAG port
log_port = log_port1
col_state = COLLECTOR_ENABLE
rc = sx_api_lag_port_collector_set(handle, lag_id, log_port, col_state)
print ("sx_api_lag_port_collector_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, col_state, rc) )

# Enables distribution on a specific LAG port
log_port = log_port1
distr_state = DISTRIBUTOR_ENABLE
rc = sx_api_lag_port_distributor_set(handle, lag_id, log_port, distr_state)
print ("sx_api_lag_port_distributor_set lag_id 0x%x , port 0x%x, state: %d,  rc %d " % (lag_id,log_port, distr_state, rc) )

# Create sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = rate
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_ADD, lag_id, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set ADD rc %d %x" % (rc, log_port) )

cntr = 0
print ("Waiting for sflow collection... ")
while (cntr < sleep_cntr):
   time.sleep(1)
   cntr = cntr + 1

print ("... finished %d " % (cntr) )

# Get sflow statistics from port
sx_port_sflow_statistics_p = new_sx_port_sflow_statistics_t_p()
sx_port_sflow_statistics = sx_port_sflow_statistics_t()
sx_port_sflow_statistics_t_p_assign(sx_port_sflow_statistics_p, sx_port_sflow_statistics)
rc = sx_api_port_sflow_statistics_get(handle, SX_ACCESS_CMD_READ, lag_id, sx_port_sflow_statistics_p)
print ("sx_api_port_sflow_statistics_get READ rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_statistics : %d " % (sx_port_sflow_statistics.count_sample_drop) )

# Delete sflow from port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_DELETE, lag_id, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set DELETE rc %d %x" % (rc, log_port) )

# Delete ports from LAG
swid = 0
sx_port_log_id_t_arr_setitem(port_list, 0, log_port1)
sx_port_log_id_t_arr_setitem(port_list, 1, log_port2)
sx_port_log_id_t_arr_setitem(port_list, 2, log_port3)
sx_port_log_id_t_arr_setitem(port_list, 3, log_port4)
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DELETE, swid, lag_id_p, port_list, 4)
print ("sx_api_lag_port_group_set DELETE ports 0x%x, 0x%x, 0x%x 0x%x from lag_id 0x%x , rc %d "
       % (log_port1, log_port2, log_port3, log_port4, lag_id, rc) )

# Destroy LAG
swid = 0
rc = sx_api_lag_port_group_set(handle, SX_ACCESS_CMD_DESTROY, swid, lag_id_p, None, 0)
lag_id = sx_port_log_id_t_p_value(lag_id_p)
print ("sx_api_lag_port_group_set DESTROY lag_id 0x%x , rc %d " % (lag_id, rc) )

# Cleanup allocated objects 
delete_sx_port_log_id_t_p(lag_id_p)
delete_sx_port_log_id_t_arr(port_list)

print "[+] end test"

