#!/usr/bin/env python
'''
This file contains a Python commands example for the sflow module.

Python commands syntax is very similar to the Switch SDK APIs.

You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

\section cases Example Flow

The example below performs the following configuration operations:

1. Open host intreface.

2. Set trap group.

3. Set trap id for sflow.

4. Register trap with host interface.

5. Create sflow on port

6. Delete sflow from port

7. Create sflow on port

8. Delete sflow from port

9. Create sflow on port

10. Edit sflow on port

11. Edit sflow on port

12. Edit sflow on port

13. Read sflow on port

14. Delete sflow from port

15. Create sflow on port

16. Read sflow statistics on port

17. Read and clear sflow statistics on port

18. Clear sflow statistics on port

19. Delete sflow from port

'''
import sys, errno
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')

import time
import os
import sxd_api
from sx_api import *

MIN_IDX = 2
MAX_IDX = 8

# SX_TRAP_ID_ETH_L2_PACKET_SAMPLING
# SX_TRAP_ACTION_MIRROR_2_CPU
# SX_TRAP_PRIORITY_BEST_EFFORT
# sx_api_host_ifc_open 1
# sx_api_host_ifc_trap_group_set 0 1 {SX_TRAP_PRIORITY_BEST_EFFORT;SX_TRUNCATE_MODE_DISABLE;0}
# sx_api_host_ifc_trap_id_set 0 SX_TRAP_ID_ETH_L2_PACKET_SAMPLING 1 SX_TRAP_ACTION_TRAP_2_CPU
# sx_api_host_ifc_trap_id_register_set SX_ACCESS_CMD_REGISTER 0 SX_TRAP_ID_ETH_L2_PACKET_SAMPLING {SX_USER_CHANNEL_TYPE_FD;1}

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % ( handle, rc) )
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

# Open host intreface
sx_fd_p = new_sx_fd_t_p()
rc = sx_api_host_ifc_open(handle, sx_fd_p)
sx_fd = sx_fd_t_p_value(sx_fd_p)
print ("sx_api_host_ifc_open 0x%x , rc %d " % ( sx_fd.fd , rc) )

# Set trap group
swid = 0
trap_group = 1
# SX_TRAP_ACTION_TRAP_2_CPU
sx_trap_group_attributes_p = new_sx_trap_group_attributes_t_p()
sx_trap_group_attributes = sx_trap_group_attributes_t()
sx_trap_group_attributes.prio = SX_TRAP_PRIORITY_BEST_EFFORT
sx_trap_group_attributes.truncate_mode = SX_TRUNCATE_MODE_DISABLE
sx_trap_group_attributes.truncate_size = 0
sx_trap_group_attributes.control_type = SX_CONTROL_TYPE_DEFAULT
sx_trap_group_attributes_t_p_assign(sx_trap_group_attributes_p, sx_trap_group_attributes)
rc = sx_api_host_ifc_trap_group_set(handle, swid, trap_group, sx_trap_group_attributes_p)
print ("sx_api_host_ifc_trap_group_set prio 0x%x , rc %d " % ( sx_trap_group_attributes.prio , rc) )

# Set trap id for sflow
trap_id = SX_TRAP_ID_ETH_L2_PACKET_SAMPLING
trap_action = SX_TRAP_ACTION_TRAP_2_CPU
rc = sx_api_host_ifc_trap_id_set(handle, swid, trap_id, trap_group, trap_action)
print ("sx_api_host_ifc_trap_id_set rc %d " % (rc) )

# Register trap with host interface
sx_user_channel_p = new_sx_user_channel_t_p()
sx_user_channel = sx_user_channel_t()
sx_user_channel.type = SX_USER_CHANNEL_TYPE_FD
sx_user_channel.channel.fd = sx_fd
sx_user_channel_t_p_assign(sx_user_channel_p, sx_user_channel)
rc = sx_api_host_ifc_trap_id_register_set(handle, SX_ACCESS_CMD_REGISTER, swid, trap_id, sx_user_channel_p)
print ("sx_api_host_ifc_trap_id_register_set  rc %d " % (rc) )

# Create sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10100
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_ADD, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set ADD rc %d %x" % (rc, log_port) )

# Delete sflow from port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params.ratio = 1
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10100
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_DELETE, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set DELETE rc %d %x" % (rc, log_port) )

# Create sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1000
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x12100
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_ADD, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set ADD rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Delete sflow from port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params.ratio = 1
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x12100
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_DELETE, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set DELETE rc %d %x" % (rc, log_port) )

# Create sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1000
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_ADD, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set ADD rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Edit sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 2000
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_EDIT, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set EDIT rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Edit sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1000
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_EDIT, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set EDIT rc %d port %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Edit sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 500
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_EDIT, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set EDIT rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Read sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 0
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_get(handle, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_get READ rc %d %x" % (rc, log_port) )

# Delete sflow from port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x10300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_DELETE, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set DELETE rc %d %x" % (rc, log_port) )

# Create sflow on port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1000
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x12300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_ADD, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set ADD rc %d %x" % (rc, log_port) )
print ( "sx_port_sflow_params : %x %x " 
	% (sx_port_sflow_params.ratio,
	sx_port_sflow_params.deviation) )

# Read sflow statistics on port
log_port = 0x12300
sx_port_sflow_statistics_p = new_sx_port_sflow_statistics_t_p()
sx_port_sflow_statistics = sx_port_sflow_statistics_t()
sx_port_sflow_statistics_t_p_assign(sx_port_sflow_statistics_p, sx_port_sflow_statistics)
rc = sx_api_port_sflow_statistics_get(handle, SX_ACCESS_CMD_READ, log_port, sx_port_sflow_statistics_p)
print ("sx_api_port_sflow_statistics_get READ rc %d %x" % (rc, log_port) )

# Read and clear sflow statistics on port
log_port = 0x12300
sx_port_sflow_statistics_p = new_sx_port_sflow_statistics_t_p()
sx_port_sflow_statistics = sx_port_sflow_statistics_t()
sx_port_sflow_statistics_t_p_assign(sx_port_sflow_statistics_p, sx_port_sflow_statistics)
rc = sx_api_port_sflow_statistics_get(handle, SX_ACCESS_CMD_READ_CLEAR, log_port, sx_port_sflow_statistics_p)
print ("sx_api_port_sflow_statistics_get READ_CLEAR rc %d %x" % (rc, log_port) )

# Clear sflow statistics on port
log_port = 0x12300
sx_port_sflow_statistics_p = new_sx_port_sflow_statistics_t_p()
sx_port_sflow_statistics = sx_port_sflow_statistics_t()
sx_port_sflow_statistics_t_p_assign(sx_port_sflow_statistics_p, sx_port_sflow_statistics)
rc = sx_api_port_sflow_statistics_get(handle, SX_ACCESS_CMD_CLEAR, log_port, sx_port_sflow_statistics_p)
print ("sx_api_port_sflow_statistics_get CLEAR rc %d %x" % (rc, log_port) )

# Delete sflow from port
sx_port_sflow_params_p = new_sx_port_sflow_params_t_p()
sx_port_sflow_params = sx_port_sflow_params_t()
sx_port_sflow_params.ratio = 1
sx_port_sflow_params.deviation = 0
sx_port_sflow_params.packet_types.uc = 0
sx_port_sflow_params.packet_types.mc = 0
sx_port_sflow_params.packet_types.bc = 0
sx_port_sflow_params.packet_types.uuc = 0
sx_port_sflow_params.packet_types.umc = 0
log_port = 0x12300
sx_port_sflow_params_t_p_assign(sx_port_sflow_params_p, sx_port_sflow_params)
rc = sx_api_port_sflow_set(handle, SX_ACCESS_CMD_DELETE, log_port, sx_port_sflow_params_p)
print ("sx_api_port_sflow_set DELETE rc %d %x" % (rc, log_port) )
